#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TStopwatch.h"

#ifndef __CINT__
#include <cstdio>
#include <ctime>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#endif

int num_events = 50000;
struct RawData  {
  int NumBytes;
  unsigned char* Data; //[NumBytes]
  RawData() : NumBytes(0), Data(0) {}
  virtual ~RawData() {}
};

struct IOBuff  {
  FILE* input;
  std::vector<std::pair<int, unsigned char*> > data;
  size_t cursor;
  IOBuff(const char* fname) : cursor(0) {
    input = fopen(fname,"rb");
    std::pair<int, unsigned char*> e;
    int len = 1;
    while(len>0)  {
      len = fread(&e.first, 4, 1, input);
      if ( len > 0 )  {
        e.second = new unsigned char[e.first];
        len = fread(e.second, e.first, 1, input);
        if ( len > 0 )  {
          data.push_back(e);
        }
      }
    }
    printf("Read %d events in total.\n\n",data.size());
  }
  std::pair<int, unsigned char*> next()  {
    size_t c = cursor;
    if ( ++cursor > data.size()-1 ) cursor=0;
    return data[c];
  }
  ~IOBuff() {
    fclose(input);
  }
};
// int buff_read(void* buffer, size_t count, size_t size);

#ifndef __CINT__

int writeFile(IOBuff& input, const char* fname)  {
  TFile *f    = new TFile(fname, "RECREATE", "LHCb RAW data test", 0);
  TTree *tree = new TTree("RawData","RAW data ROOT tree");
  RawData* obj = new RawData();
  TBranch *bData = tree->Branch("Data", "RawData", &obj, 32000*20, 99);
  TStopwatch timer;
  timer.Start();
  for(int i=0, cnt=0; i < num_events; ++i) {
    std::pair<int, unsigned char*> e = input.next();
    obj->NumBytes = e.first;
    obj->Data     = e.second;
    cnt += tree->Fill();
    if ( (i%1000)==0 )
    {
      timer.Stop();
      printf("Wrote %5d Events [%f [CPU] %f [RT] seconds %7.2f Hz [CPU] %7.2f Hz [RT] ] : %d\n",i, float(timer.CpuTime()), float(timer.RealTime()), float(i)/float(timer.CpuTime()+1e-6), float(i)/float(timer.RealTime()+1e-6), cnt);
      timer.Start(kFALSE);
    }
  }
  timer.Stop();
  bData->Write();
  tree->Print();
  tree->Write();
  f->Print();
  f->Close();
  delete f;
  unlink(fname);
  return 1;
}

int writeRaw(IOBuff& input, const char* fname)  {
  int output = open(fname,O_CREAT|O_TRUNC|O_RDWR);
  TStopwatch timer;
  timer.Start();
  for(int i=0, cnt = 0; i < num_events; ++i) {
    std::pair<int, unsigned char*> e = input.next();
    cnt += ::write(output, &e.first, 4);
    cnt += ::write(output, e.second, e.first);
    if ( (i%1000)==0 )    {
      timer.Stop();
      printf("Wrote %5d Events [%f [CPU] %f [RT] seconds %7.2f Hz [CPU] %7.2f Hz [RT] ] : %d\n",i, float(timer.CpuTime()), float(timer.RealTime()), float(i)/float(timer.CpuTime()+1e-6), float(i)/float(timer.RealTime()+1e-6), cnt);
      timer.Start(kFALSE);
    }
  }
  timer.Stop();
  close(output);
  unlink(fname);
  return 1;
}
#include <cerrno>

int writeIOV(IOBuff& input, const char* fname)  {
#ifndef _WIN32
  int buff_len = 32*1024*1024;
  int fdesc = open(fname,O_CREAT|O_TRUNC|O_RDWR, S_IRUSR|S_IWUSR);
  iovec iov[2];
  TStopwatch timer;
  timer.Start();
  for(int i=0, cnt = 0; i < num_events; ++i) {
    std::pair<int,unsigned char*> e = input.next();
    iov[0].iov_len  = sizeof(int);
    iov[0].iov_base = &e.first;
    iov[1].iov_len  = e.first;
    iov[1].iov_base = e.second;
    cnt += writev(fdesc, iov, 2);
    if ( (i%1000)==0 )  {
      timer.Stop();
      printf("Wrote %5d Events [%f [CPU] %f [RT] seconds %7.2f Hz [CPU] %7.2f Hz [RT] ] : %d\n",i, float(timer.CpuTime()), float(timer.RealTime()), float(i)/float(timer.CpuTime()+1e-6), float(i)/float(timer.RealTime()+1e-6), cnt);
      timer.Start(kFALSE);
    }
  }
  timer.Stop();
  close(fdesc);
  unlink(fname);
#endif
  return 1;
}
#endif

void RawBuffer()  {
  IOBuff input("rawBuffer.dat");
  writeFile(input, "/tmp/tmpData.root");
  writeRaw(input, "/tmp/tmpData.raw");
  writeIOV(input, "tmpData.raw");
}

