#include "pool/Ref.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"
#include "POOLCore/PoolMessageStream.h"

#include <iostream>
#include "AthenaPoolTestMap.h"
#include "AthenaPoolTestMapData.h"
#include "AthenaPoolTestMapDataLink.h"

static int s_printoutMod = 100;

namespace pool  {

  class AtlasMaps : public TestBase {
  public: 

    /// Standard Constructor
    AtlasMaps() : TestBase(ROOTKEY_StorageType.type(), "AtlasMaps_1.root", "AtlasMaps_2.root")  { }

    /// Standard Destructor
    virtual ~AtlasMaps()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      sc = unlink(m_dbName2.c_str())==0 ? DbStatus::SUCCESS : DbStatus::ERROR;
      checkError(sc,"Cannot delete file: "+m_dbName2);
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      return write_objects(dbcH1, dbcH2);
    }

    DbStatus write_objects(ConnectionH dbcH1, ConnectionH dbcH2) {
      DbStatus sc;
      TransactionH trH1 = 0, trH2 = 0;
      Guid g3(getGuid("AthenaPoolTestMap"));
      Placement place1(m_dbName1,"test_cont_1", g3, m_type);
      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database transactions (1)");
      for ( size_t i = 0; i < m_innerLoops; ++i )   {
        // Fill and write out nested collections
        AthenaPoolTestMap* tmapPtr = new AthenaPoolTestMap();
        AthenaPoolTestMap& tmap    = *tmapPtr;

        typedef AthenaPoolTestMapData::Deposit deposit_t;
        typedef std::vector<AthenaPoolTestMapData::Deposit>  deposit_vec;
        deposit_vec deposits;
        deposit_t deposit;
        for (unsigned int j = 0; j < 10; ++j) {
          for (unsigned int i = 0; i < 10; ++i) {

            deposit.first  = AthenaPoolTestMapDataLink(i, i + 10);
            deposit.second = float(i + 20);
            deposits.push_back(deposit);
          }

          tmap[j*5] = AthenaPoolTestMapData(deposits, j);
          deposits.clear();
        }

        // Printout data
        AthenaPoolTestMap::const_iterator itm  = tmap.begin();
        AthenaPoolTestMap::const_iterator endm = tmap.end();
        for (; itm != endm; ++itm) {

          std::cout << "AthenaPoolTestMap: key " << (*itm).first
            << "  word " << (*itm).second.word()
            << "  Deposits: (barCode, evtIndx, wgt) ";

          deposits.clear();
          (*itm).second.deposits(deposits);
          for (unsigned int i = 0; i < deposits.size(); ++i) {
            AthenaPoolTestMapDataLink& link = deposits[i].first;
            std::cout << link.barcode() << " "
              << link.eventIndex() << " "
              << deposits[i].second << " ";
          }
          std::cout << std::endl;
        }
        pool::Ref<AthenaPoolTestMap > r(tmapPtr);
        sc = cache()->registerForWrite(trH1, place1, r.ptr());
        checkError(sc,"Error cache.registerForWrite(trH1, place1, AthenaPoolTestMap) [1]");
      }
      std::cout << "Commit transaction..." << std::endl;
      //gDebug = 3;
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]");
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->clear();
      checkError(sc,"Error cache.clear(...)");
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }

    /// read objects from the persistent store
    DbStatus read()  {
      DbStatus sc;
      Guid g3(getGuid("AthenaPoolTestMap"));
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      for(size_t i=0; i < m_innerLoops; ++i)   {
        Token *tok2 = new Token();

        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok2->setDb(m_dbName1);
        tok2->setCont("test_cont_1");
        tok2->setTechnology(m_type);
        tok2->setClassID(g3);
        tok2->oid().first  = 3;
        tok2->oid().second = i;
        // Read in nested collections
        pool::Ref<AthenaPoolTestMap> _r(cache(), tok2);

        const AthenaPoolTestMap& tmap    = *(_r.ptr());
        typedef AthenaPoolTestMapData::Deposit deposit_t;
        typedef std::vector<deposit_t>         deposit_vec;

        deposit_vec deposits;
        // Printout data
        AthenaPoolTestMap::const_iterator itm  = tmap.begin();
        AthenaPoolTestMap::const_iterator endm = tmap.end();
        for (; itm != endm; ++itm) {

          std::cout << "AthenaPoolTestMap: key " << (*itm).first
            << "  word " << (*itm).second.word()
            << "  Deposits: (barCode, evtIndx, wgt) ";

          deposits.clear();
          (*itm).second.deposits(deposits);
          for (unsigned int i = 0; i < deposits.size(); ++i) {
            AthenaPoolTestMapDataLink& link = deposits[i].first;
            std::cout << link.barcode() << " "
              << link.eventIndex() << " "
              << deposits[i].second << " ";
          }
          std::cout << std::endl;
        }
      }
      return poolDb::Success;
    }
  };
}
pool::TestRunner<pool::AtlasMaps> s_testAtlasMaps("AtlasMaps");
