#include <string>

#ifdef _WIN32
#include <windows.h>
#else
#include <errno.h>
#include <dlfcn.h>
#endif

/// Retrieve last error code
unsigned long getLastError()    {
#ifdef WIN32
  return ::GetLastError();
#else
  return errno;
#endif
}

#ifndef WIN32
namespace boost {
  class xtime;
  void xtime_get(xtime*, int) {}
}
#endif

/// Retrieve error code as string for a given error
const std::string getErrorString(unsigned long error)    {
  std::string errString =  "";
#ifdef WIN32
  LPVOID lpMessageBuffer;
  ::FormatMessage( 
    FORMAT_MESSAGE_ALLOCATE_BUFFER |  FORMAT_MESSAGE_FROM_SYSTEM,
    NULL,
    error,
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), //The user default language
    (LPTSTR) &lpMessageBuffer,
    0,
    NULL );
  errString = (const char*)lpMessageBuffer;
  // Free the buffer allocated by the system
  ::LocalFree( lpMessageBuffer ); 
#else
  // Remember: for linux dl* routines must be handled differently!
  if ( error == 0xAFFEDEAD )  {
    errString = ::dlerror();
    errno = 0;
  }
  else    {
    errString = ::strerror(error);
  }
#endif
  return errString;
}

