#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "ObjyMemoryFile.h"
#include "Pairs.h"
#include "TNamed.h"

#include "TROOT.h"
#include "TClass.h"
#include "TStreamerInfo.h"
#include "TStreamerElement.h"

static int s_printoutMod = 100;

namespace pool  {

  class CMSRefs : public TestBase {
  public: 

    /// Standard Constructor
    CMSRefs() : TestBase(ROOTKEY_StorageType.type(), "test_CMS_Ref1.root", "test_CMS_Ref2.root") 
    {   }

    /// Standard Destructor
    virtual ~CMSRefs()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      TransactionH trH1 = 0;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      Placement place1(m_dbName1,"test_cont_1", getGuid("Pairs::PairAggregate<double>"), m_type);
      std::cout << "Number of OUTER loops:" << m_innerLoops << std::endl;
      std::cout << "Number of INNER loops:" << m_outerLoops << std::endl;

      for ( size_t j = 0; j < m_innerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_outerLoops; ++i )   {
          pool::Ref<Pairs::PairAggregate<double> > ref(new Pairs::PairAggregate<double>());
          sc = cache()->registerForWrite(trH1, place1, ref.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, TNamed) [1]");
          }
        }
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]");
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"Error cache.disconnect(...) [1]");

      dumpStreamerInfo("Pairs::Complement");
      dumpStreamerInfo("Pairs::PairAggregate<double>");
      dumpStreamerInfo("pool::Ref<Pairs::Templated<double> >");
      dumpStreamerInfo("pair<Pairs::Complement,pool::Ref<Pairs::Templated<double> > >");
      return 1;
    }
  };
}

pool::TestRunner<pool::CMSRefs> s_testCMSRefs("CMSRefs");
