#include "pool/test.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbPrint.h"
#include "StorageSvc/DbReflex.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

namespace pool  {

  class Abstract : public TestBase {
  public: 

    /// Standard Constructor
    Abstract() : TestBase(ROOTKEY_StorageType.type(), "test_1_1.root", "test_1_2.root")  { }
    /// Standard Destructor
    virtual ~Abstract()    {    }

    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH = 0;
      TransactionH trH = 0;
      Placement place(m_dbName1,"test_cont_1", getGuid("testSimple"), m_type);
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      accessRootClass("testSimple");

      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH, trH);
        checkError(sc,"Cannot start database transactions", true);
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          printRound(i, s_printoutMod);
          pool::Ref<testSimple> refObj(new testSimple);
	  refObj->m_abstract = new AbstractImp();

          sc = cache()->registerForWrite(trH, place, refObj.ptr());
          checkError(sc,"Error cache.registerForWrite(trH, place, test)", true);
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH);
        checkError(sc,"Error cache.commit(...)", true);
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH);
      checkError(sc,"Error cache.disconnect(...)", true);
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      TransactionH trH = 0;
      ConnectionH  dbcH = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      sc = cache()->startTransaction(dbcH, trH);
      checkError(sc,"Cannot start database transactions", true);
      accessRootClass("testSimple");
      dumpStreamerInfo("AbstractImp");
      dumpStreamerInfo("AbstractBase");
      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *token = new Token();
        token->setDb(m_dbName1);
        token->setCont("test_cont_1");
        token->setTechnology(m_type);
        token->setClassID(getGuid("testSimple"));
        token->oid().first  = 3;
        token->oid().second = i;
        printRound(i, s_printoutMod);
        DbPrint log("TestAbstract");
        log << DbPrintLvl::Info;

        // Read first object
        pool::Ref<testSimple> refObj(cache(), token);
        if ( refObj ) {
          token = token;
        }
        if ( refObj && i%s_printoutMod == 0 )   {
          log << (void*)refObj->m_abstract << DbPrint::endmsg;
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "test MISSING";
        }
        cache()->clear();
      }
      sc = cache()->commit(trH);
      checkError(sc,"Error cache.commit(...)", true);
      std::cout << "Clear cache..." << std::endl;
      cache()->clear();
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::Abstract> s_testAbstract("Abstract");
