#include "StorageSvc/DbSession.h"
#include "StorageSvc/DbDomain.h"
#include "StorageSvc/DbDatabase.h"
#include "StorageSvc/DbContainer.h"
#include "StorageSvc/DbTransaction.h"
#include "StorageSvc/DbObjectCallBack.h"
#include "StorageSvc/DbTransform.h"
#include "StorageSvc/DbTypeInfo.h"
#include "StorageSvc/DbColumn.h"
#include "StorageSvc/DbObject.h"
#include "StorageSvc/DbIter.h"
#include "StorageSvc/DbHeap.h"
#include "POOLCore/Token.h"

#include "pool/TestBase.h"
#include "pool/rawio.h"
#include "pool/test.h"
#include "TNamed.h"
#include <iostream>

using namespace pool;

namespace pool  {
  class RawIO : public TestBase {
  public:
    std::string m_domName;

    /// Standard Constructor
    RawIO() : TestBase(ROOTKEY_StorageType.type(), "raw_io.root", "", false)     {
      m_domName = "Root Event Data";
      m_innerLoops = 10000;
    }

    /// Standard Destructor
    virtual ~RawIO()               {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc = DbStatus::Success;
#ifdef __RAW_IO
      unlink(m_dbName1.c_str());

      char txt[128];
      DbSession   sesH;
      if ( sesH.open(0).isSuccess() )   {
        DbTypeInfo *typ1 = 0, *typ2 = 0, *typ3 = 0, *typ4 = 0;
        {
          std::vector<const DbColumn*> cols;
          test* t1 = (test*)0;
          cols.push_back(new DbColumn("b",          // field name
                                      DbColumn::USHORT,
                                      long(&t1->m_ushort),  // field offset
                                      sizeof(t1->m_ushort), // field size(length)
                                      1,
                                      0));
          cols.push_back(new DbColumn( "c",DbColumn::INT,long(&t1->m_int),sizeof(t1->m_int),1,0));
          cols.push_back(new DbColumn( "d",DbColumn::LONG,long(&t1->m_long),sizeof(t1->m_long),1,0));
          cols.push_back(new DbColumn( "s",DbColumn::STRING,long(&t1->m_string),sizeof(t1->m_string),1,0));
          cols.push_back(new DbColumn("Link.first",DbColumn::INT,long(&t1->m_linkH.first),sizeof(&t1->m_linkH.first),1,0));
          cols.push_back(new DbColumn("Link.second",DbColumn::INT,long(&t1->m_linkH.second),sizeof(&t1->m_linkH.second),1,0));
          cols.push_back(new DbColumn("Links",DbColumn::TOKEN,long(&t1->m_links),sizeof(&t1->m_links),1,0));
          typ1 = DbTypeInfo::create("test", cols);
        }
        typ2 = typ1;
        typ3 = DbTypeInfo::create("TNamed");
        typ4 = typ3;

        const DbTypeInfo* types[] = {typ1, typ2, typ3, typ4, 0};
        const char* containerNames[] = { "Cnt_test__1", 
                                         "Cnt_test__2", 
                                         "Cnt_TNamed__1",
                                         "Cnt_TNamed__2",
                                         0};
        DbDomain domH(m_type);
        sc = domH.open(sesH, domH.type(), poolDb::UPDATE|poolDb::CREATE);
        if ( sc.isSuccess() )   {
          DbDatabase  dbH(domH.type());
          sc = dbH.open(domH, m_dbName1, m_dbName1, poolDb::CREATE);
          if ( sc.isSuccess() )   {
            Token tok;
            for (size_t i=0, j=0, k=0; types[i]; ++i )  {
              std::cout << "+----------------------------------------------------+" << std::endl;
              std::cout << "|  " << containerNames[i] << std::endl;
              std::cout << "+----------------------------------------------------+" << std::endl;
              const DbTypeInfo* typ = types[i];
              sc = dbH.addShape(typ);
              if ( !sc.isSuccess() )   {
                std::cout << "+----------------------------------------------------+" << std::endl;
                std::cout << "|  CANNOT ADD SHAPE:" << containerNames[i] << std::endl;
                std::cout << "+----------------------------------------------------+" << std::endl;
              }
              else  {
                std::string cntName = containerNames[i];
                      std::string tn = typ->shapeID().toString();
                const DbTypeInfo* typH = dbH.objectShape(typ->shapeID());
                if ( !typH )   {
                  std::cout << "+----------------------------------------------------+" << std::endl;
                  std::cout << "|  CANNOT RETRIEVE SHAPE:" << containerNames[i] << std::endl;
                  std::cout << "+----------------------------------------------------+" << std::endl;
                }
                else  {
                  DbContainer cntH(dbH.type());
                  sc = cntH.open(dbH, cntName, typH, m_type, poolDb::CREATE);
                  if ( !sc.isSuccess() )   {
                    std::cout << "+----------------------------------------------------+" << std::endl;
                    std::cout << "|  CANNOT OPEN:" << containerNames[i] << std::endl;
                    std::cout << "+----------------------------------------------------+" << std::endl;
                  }
                  else  {
                    DbTransaction tr;
                    sc = cntH.transAct(tr);
                    if ( sc.isSuccess() )   {
                      std::cout << "+----------------------------------------------------+" << std::endl;
                      if ( i == 0 )  {
                        cntH.token()->set(&tok);
                        std::cout << "|   Reference token: " << tok.key() << std::endl;
                      }
                      else  {
                        std::cout << "|  Populating: " << containerNames[i] << std::endl;
                      }
                      std::cout << "+----------------------------------------------------+" << std::endl;

                      for (j = 0; j < m_innerLoops; ++j )   {
                        if ( i == 0 )   {
                          ::sprintf(txt,"test_%s_%d",containerNames[i],int(j));
                          DbHandle<test> objH = new(cntH, typH) test(txt);
                          objH->m_string = txt;
                          objH->m_short  = j;
                          objH->m_int    = 100*j;
                          objH->m_long   = 10000*j;
                          for (k=0; k < (j%10); ++k )  {
                            objH->m_links.push_back(DbLink(k,100*k));
                          }
                        }
                        else if ( i == 1 )   {
                          ::sprintf(txt,"test_%s_%d",containerNames[i],int(j));
                          DbHandle<test> objH = new(cntH, typH) test(txt);
                          objH->m_string = txt;
                          objH->m_short  = j;
                          objH->m_int    = 100*j;
                          objH->m_long   = 10000*j;
                          DbLink& lnk = const_cast<DbLink&>(tok.oid());
                          lnk.second = j;
                          objH.makeLink(&tok, objH->m_linkH);
                        }
                        else if ( i == 2 )   {
                          ::sprintf(txt,"TNamed_%s_%d",containerNames[i],int(j));
                          DbHandle<TNamed > objH = ::new(cntH, typH) TNamed(txt,"Named");
                        }
                        else if ( i == 3 )   {
                          DbLink lnk;
                          ::sprintf(txt,"TNamed_%s_%d",containerNames[i],int(j));
                          DbObjectCallBack* m = new DbObjectCallBack(typH->clazz());
                          m->setObject(new TNamed(txt,"Named"));
                          m->setShape(typ);
                          cntH.allocate(m, lnk);
                        }
                        if ( !sc.isSuccess() ) break;
                      }
                      std::cout << "DbObject instances: " << DbHeap::numInstances() << std::endl;
                      std::cout << "DbContainer reference count:" << cntH.refCount() << std::endl;
                      sc = cntH.transAct(tr.set(Transaction::TRANSACT_COMMIT));
                      std::cout << "Wrote " << j << " objects of type " << containerNames[i] << std::endl;
                      std::cout << "DbContainer reference count:" << cntH.refCount() << std::endl;
                      std::cout << "DbObject instances: " << DbHeap::numInstances() << std::endl;
                    }
                    cntH.close();
                    std::cout << "Database  reference count:" << dbH.refCount() << std::endl;
                  }
                }
              }
            }
            std::cout << "DataBase reference count:" << dbH.refCount() << std::endl;
            std::cout << "Domain   reference count:" << domH.refCount() << std::endl;
            dbH.close();
            std::cout << "Domain   reference count:" << domH.refCount() << std::endl;
            std::cout << "Session  reference count:" << sesH.refCount() << std::endl;
            domH.close();
            std::cout << "Session  reference count:" << sesH.refCount() << std::endl;
            sesH.close();
          }
        }
      }
#endif
      return sc;
    }

    /// read objects from the persistent store
    DbStatus read()  {
      DbStatus sc = DbStatus::Success;
#ifdef __RAW_IO
      DbSession   sesH;
      if ( sesH.open(0).isSuccess() )   {
        const char* containerNames[] = { "Cnt_test__1", 
                                         "Cnt_test__2", 
                                         "Cnt_TNamed__1",
                                         "Cnt_TNamed__2",
                                         0};
        DbDomain domH(m_type);
        sc = domH.open(sesH, domH.type(), poolDb::READ);
        if ( sc.isSuccess() )   {
          DbDatabase  dbH(domH.type());
          sc = dbH.open(domH, m_dbName1, m_dbName1);
          if ( sc.isSuccess() )   {
            // Have a brief look what containers are contained in the database
            std::vector<const Token*> conts;
            std::vector<const Token*>::iterator ci;
            sc = dbH.containers(conts);
            std::cout << "+-------- Container overview: " << std::endl;
            for ( ci=conts.begin(); ci != conts.end(); ++ci)  {
              std::cout << "| " << (*ci)->dbID() << "::" << (*ci)->contID() << std::endl;
            }
            sc = dbH.associations(conts);
            std::cout << "+-------- Association overview: " << std::endl;
            for ( ci=conts.begin(); ci != conts.end(); ++ci)  {
              std::cout << "| " << (*ci)->dbID() << "::" << (*ci)->contID() << std::endl;
            }
            // Loop over data items...
            for ( int jj = 0; jj < 1; ++jj ) {
              for ( int i=0; containerNames[i]; ++i )  {
                std::string cntName = containerNames[i];
                const DbTypeInfo* typ = dbH.contShape(cntName);
                if ( typ )    {
                  ClassH cl = typ->clazz();
                  if ( !cl )    {
                    std::cout << "Unknown class of container:" << cntName << std::endl;
                  }

                  DbContainer cntH(dbH.type());
                  sc = cntH.open(dbH, cntName, typ, m_type, poolDb::READ);
                  int cnt = 0;
                  if ( sc.isSuccess() )   {
                    if ( i == 0 )   {
                      DbIter<test> it;
                      for ( it.scan(cntH, typ); it.next().isSuccess(); ++cnt )   {
                        DbLink oid = (*it).oid();
                        if ( cnt < 10 )   {
                          std::cout << "OID: [" << oid.first << "," << oid.second << "] DATA:"
                                    << it->m_short << " " << it->m_int << " " << it->m_long << " " << it->m_string;
                          std::cout << " [" << it->m_links.size() << "] ";
                          for(size_t k=0; k < it->m_links.size(); ++k )  {
                            std::cout << "(" << it->m_links[k].first 
                                      << "," << it->m_links[k].second << ") ";
                          }
                          std::cout << std::endl;
                        }
                      }
                    }
                    else if ( i == 1 )   {
                      DbIter<test> it;
                      for ( it.scan(cntH, typ); it.next().isSuccess(); ++cnt)   {
                        test* tst = *it;
                        DbHandle<test> objH = *it;
                        DbLink oid = (*it).oid();
                        sc = objH.open(objH.containedIn(), tst->m_linkH);
                        if ( cnt < 10 )   {
                          std::cout << "OID: [" << oid.first << "," << oid.second << "] DATA:"
                                    << tst->m_short << " " << tst->m_int << " " << tst->m_long << " " << tst->m_string 
                                    << " Link: (" << tst->m_linkH.first << " , " << tst->m_linkH.second << ")"
                                    << std::endl;
                          std::cout << "   --->" << objH->m_short
                                    << " " << objH->m_int
                                    << " " << objH->m_long
                                    << " " << objH->m_string;
                          std::cout << " [" << objH->m_links.size() << "] ";
                          for(size_t k=0; k < objH->m_links.size(); ++k )  {
                            std::cout << "(" << objH->m_links[k].first 
                                      << "," << objH->m_links[k].second << ") ";
                          }
                          std::cout << std::endl;
                        }
                      }
                    }
                    else if ( i == 2 )   {
                      DbObjectCallBack  cb(cl);
                      DbPtrIter<TNamed> it(cb);
                      it.call()->setShape(typ);
                      for ( it.scan(cntH, typ); it.next().isSuccess(); ++cnt)   {
                        TNamed* tst = (TNamed*)it.object();
                        if ( cnt < 10 ) {
                          std::cout << tst->GetName() << " " << tst->GetTitle() << std::endl;
                        }
                      }
                    }
                    else if ( i == 3 )   {
                      DbObjectCallBack  cb(cl);
                      DbPtrIter<TNamed> it(cb);
                      it.call()->setShape(typ);
                      for ( it.scan(cntH, typ); it.next().isSuccess(); ++cnt)   {
                        if ( cnt < 10 ) {
                          std::cout << (*it)->GetName() << " " << (*it)->GetTitle() << std::endl;
                        }
                      }
                    }
                  }
                  std::cout << "Read " << cnt << " objects of container " << containerNames[i] << std::endl;
                }
              }
            }
            dbH.close();
            dbH = DbDatabase(domH.type());
          }
          domH.close();
          domH = DbDomain(m_type);
        }
        sesH.close();
      }
#endif
      return sc;
    }
  };
}

pool::TestRunner<pool::RawIO> s_RawIO("RawIO");
