#include "clhep_tests.h"
#include "pool/Ref.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

namespace pool  {

  class TestClhep : public TestBase {
  public: 

    /// Standard Constructor
    TestClhep() : TestBase(ROOTKEY_StorageType.type(), "TestClhep_1.root", "TestClhep_2.root")  {
    }

    /// Standard Destructor
    virtual ~TestClhep()    {    }

    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      TransactionH trH1 = 0;
      Placement place1(m_dbName1,"test_cont_1", getGuid("clhep_test"), m_type);
      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<clhep_test>   refObj(new clhep_test);
          refObj->fillData(i);
          sc = cache()->registerForWrite(trH1, place1, refObj.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place1, clhep_test)   [2]");
          if ( refObj && i%s_printoutMod == 0 )   {
            refObj->printData(i, false);
          }
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }

    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      Guid g3(getGuid("test"));
      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok = new Token();
        tok->setDb(m_dbName1);
        tok->setCont("test_cont_1");
        tok->setTechnology(m_type);
        tok->setClassID(g3);
        tok->oid().first  = 4;
        tok->oid().second = (3*i)+1;

        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
          cache()->setPrintout(1);
        }
        else {
          cache()->setPrintout(0);
        }
        // Read first object
        pool::Ref<clhep_test> refObj(cache(), tok);
        if ( refObj && i%s_printoutMod == 0 )   {
          refObj->printData(i,true);
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "test MISSING";
        }
        cache()->clear();
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::TestClhep> s_testTestClhep("Clhep");
