#include "pool/Ref.h"
#include "pool/test.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

namespace pool {
  void genMD5(const std::string& s, void* code);
}

namespace pool  {

  class NoVirtualTest : public TestBase {
    Guid m_guidNV;
    Guid m_guidNVS;
  public: 
    /// Standard Constructor
    NoVirtualTest() : TestBase(m_type, "NoVirtual_1.root", "NoVirtual_2.root")  {
      pool::genMD5("NoVirtual",&m_guidNV.Data1);
      pool::genMD5("NoVirtualSubClass",&m_guidNVS.Data1);      
    }

    /// Standard Destructor
    virtual ~NoVirtualTest()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      TransactionH trH1 = 0;
      Placement place1(m_dbName1,"test_cont1", m_guidNV,  m_type);
      Placement place2(m_dbName1,"test_cont2", m_guidNVS, m_type);
      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<NoVirtual> r1(new NoVirtual());
          r1->m_data1 = i;
          sc = cache()->registerForWrite(trH1, place1, r1.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place1, NoVirtual) [6]", true);

          pool::Ref<NoVirtualSubClass> r2(new NoVirtualSubClass());
          r2->m_data1 = i;
          r2->m_data2 = 100*i;
          sc = cache()->registerForWrite(trH1, place2, r2.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place2, NoVirtualSubClass) [6]", true);
        }
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]", true);
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)", true);
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]", true);
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      // Guid g6(getGuid("EventHeader"));
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);

      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok1 = new Token();
        tok1->setDb(m_dbName1);
        tok1->setCont("test_cont1");
        tok1->setTechnology(m_type);
        tok1->setClassID(m_guidNV);
        tok1->oid().first  = 3;
        tok1->oid().second = i;

        Token *tok2 = new Token();
        tok2->setDb(m_dbName1);
        tok2->setCont("test_cont2");
        tok2->setTechnology(m_type);
        tok2->setClassID(m_guidNVS);
        tok2->oid().first  = 4;
        tok2->oid().second = i;
        printRound(i, s_printoutMod);

        pool::Ref<NoVirtual> r1(cache(), tok1);
        pool::Ref<NoVirtualSubClass> r2(cache(), tok2);
        if ( r1 && i%s_printoutMod == 0 )   {
          std::cout << " r1 valid[" << i << "]:" << r1->m_data1;
        }
        else if ( i%s_printoutMod == 0 )   {
          checkError(DbStatus::Error,"r1 MISSING", true);
        }
        if ( r2 && i%s_printoutMod == 0 )   {
          std::cout << " r2 valid[" << i << "]:" << r2->m_data1 << " " << r2->m_data2;
        }
        else if ( i%s_printoutMod == 0 )   {
          checkError(DbStatus::Error,"r2 MISSING", true);
        }
        if ( i%s_printoutMod == 0 )   {
          std::cout << std::endl;
        }
        cache()->clear();
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::NoVirtualTest> s_testNoVirtualTest("NoVirtual");
