#include "Embedded_objects.h"
#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include <iostream>
#include "pool/TestBase.h"

/*

rootcint Embedded_dict.cpp -c Embedded_objects.h Embedded_LinkDef.h
g++ Embedded_dict.cpp Embedded_objects.cpp -L${ROOTSYS}/lib -lCint -lCore -I${ROOTSYS}/include -ldl -fPIC -shared -o libEmb.so

cl.exe /c /DLL /MD /Z7 /GR /GX Embedded_dict.cpp Embedded_objects.cpp /I"%ROOTSYS%/include"
link.exe Embedded_dict.obj Embedded_objects.obj /libpath:"%ROOTSYS%\lib" libCint.lib libCore.lib /DLL -out:libEmb.dll
*/

namespace pool  {
  static int counter = 0;
  struct Embedded : public TestBase {

    /// Standard Constructor
    Embedded()    : TestBase(0, "", "")     {    }

    /// Standard Destructor
    virtual ~Embedded()               {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      //return write1();
      return write1();
    }
    DbStatus write1()       {
      //gSystem->Load("test.dll");
      // gSystem->Load("libEmb.dll");
      TFile *f    = new TFile("Embedded.root", "RECREATE", "Root Embedded test");
      TTree *tree = new TTree("T","An example of a ROOT tree");
      Normal_objects* obj = new Normal_objects();
      TBranch *b = tree->Branch("B", "Normal_objects", &obj);
      for(int i=0; i < 10; ++i) {obj->initData(i);printf("%d\n",tree->Fill());}
      b->Print();
      b->Write();
      tree->Print();
      tree->Write();
      f->Print();
      f->Close();
      delete f;
      return 1;
    }
    DbStatus write2()       {
      // gSystem->Load("libEmb.so");
      TFile *f    = new TFile("Embedded2.root", "RECREATE", "Root Embedded test");
      TTree *tree = new TTree("T","An example of a ROOT tree");
      Long64_t_Container* obj = new Long64_t_Container();
      TBranch *b = tree->Branch("B", "Long64_t_Container", &obj);
      for(int i=0; i < 10; ++i) {obj->i=i; obj->ui=100*i; obj->ld=(long double)(10000.0*i); printf("%d\n",tree->Fill());}
      b->Print();
      b->Write();
      tree->Print();
      tree->Write();
      f->Print();
      f->Close();
      delete f;
      return 1;
    }
    /// write objects to the persistent store
    DbStatus read()       {
      //return read1();
      return read1();
    }
    /// read objects from the persistent store
    DbStatus read1()  {
      //gSystem->Load("Persistency_1.dll");
      //gSystem->Load("test.dll");
      TFile *f    = new TFile("Embedded.root");
      TTree *tree = (TTree*)f->Get("T");
      Normal_objects* obj = new Normal_objects();
      TBranch *branch  = tree->GetBranch("B");
      branch->SetAddress(&obj);
      Int_t nevent = Int_t(tree->GetEntries());
      Int_t nselected = 0;
      Int_t nb = 0;
      for (Int_t i=0;i<nevent;i++) { printf("%d %d\n", tree->GetEvent(i), obj); obj->dump(); }
      dumpClass("Embedded_objects");
      dumpClass("Embedded_objects::EmbeddedClasses");
      dumpClass("Embedded_objects::EmbeddedClasses::Embedded1");
      dumpClass("Embedded_objects::EmbeddedClasses::Embedded2");
      dumpClass("Embedded_objects::EmbeddedClasses::Embedded3");
      return 1;
    }
    /// read objects from the persistent store
    DbStatus read2()  {
      //gSystem->Load("Persistency_1.dll");
      // gSystem->Load("libEmb.so");
      TFile *f    = new TFile("Embedded2.root");
      TTree *tree = (TTree*)f->Get("T");
      Long64_t_Container* obj = new Long64_t_Container();
      TBranch *branch  = tree->GetBranch("B");
      branch->SetAddress(&obj);
      Int_t nevent = Int_t(tree->GetEntries());
      //for (Int_t i=0;i<nevent;i++) {cout << tree->GetEvent(i) << obj << " " << obj->i << " " << obj->ui << " " << obj->ld << endl; }
      for (Int_t i=0;i<nevent;i++) { 
        int nb=tree->GetEvent(i); 
        std::cout << nb << " " << obj 
                  << " Data:"  << obj->i 
                  << " "       << obj->ui 
                  << std::endl;
        printf("%d %d Data:%d %d\n", nb, obj, obj->i, obj->ui); 
      }
      return 1;
    }
  };
}

pool::TestRunner<pool::Embedded> s_testEmbedded("Embedded");
