#include "pool/TestBase.h"
#include "RootSizeObj.h"
#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TStreamerInfo.h"

#include <cstdlib>
#include <ctime>

/*

rootcint RootSizeObj_dict.cpp -c RootSizeObj.h RootSizeObj_LinkDef.h

cl.exe /c /DLL /MD /Z7 /GR /GX RootSizeObj_dict.cpp Embedded_objects.cpp /I"%ROOTSYS%/include"
link.exe Embedded_dict.obj Embedded_objects.obj /libpath:"%ROOTSYS%\lib" libCint.lib libCore.lib /DLL /DEBUG -o libEmb.dll
*/

RootSizeObj::RootSizeObj()
{
}

RootSizeObj::RootSizeObj(int size)
{
  static bool first = true;
  if ( first )  {
    first = false;
    srand( 0xFEEDBABE );
  }
  m_vector.reserve(size);
  for ( int i = 0; i < size; ++i )
    m_vector.push_back(rand());
}

RootSizeObj::~RootSizeObj()
{
}

namespace pool  {
  static int counter = 0;
  struct RootSizePerformance : public TestBase {

    /// Standard Constructor
    RootSizePerformance()    : TestBase(0, "" , "")     {    }

    /// Standard Destructor
    virtual ~RootSizePerformance()               {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      TFile *f = new TFile("RootSizePerformance.root", "RECREATE", "Root size performance test");
      size_t i, j, k, l;
      for ( l = 0; l < 13; ++l )   {
        char txt[64];
        std::vector<RootSizeObj*> objects;
        sprintf(txt,"cont_%d_%d", l, m_objSize);
        for ( k = 0; k < m_innerLoops; ++k )   {
          objects.push_back(new RootSizeObj(m_objSize));
        }

        // ==================================================
        clock_t start = clock();
        TTree *tree = new TTree(txt,"An example of a ROOT tree");
        TBranch *b = tree->Branch(txt, "RootSizeObj", &objects[0]);
        for ( j = 0; j < m_outerLoops*m_innerLoops; ++j )   {
          i = j%m_innerLoops;
          RootSizeObj* pObj = objects[i];
          b->SetAddress(&pObj);
          ((TTree*)f->Get(txt))->Fill();
          //tree->Fill();
        }
        clock_t stop = clock();
        std::cout << " [" << std::setw(4) << j << "] " 
                  << std::setw(10) << float(stop-start)/CLOCKS_PER_SEC 
                  << " " << std::setw(6) << m_objSize*sizeof(int) << " Bytes/Object " 
                  << std::setw(7) << j << " Objects " 
                  << " Container:" 
                  << txt << std::endl;

        for ( k = 0; k < m_innerLoops; ++k )   {
          delete objects[k];
        }
        m_objSize = m_objSize*2;
        tree->Write();
        delete tree;
      }
      f->Close();
      return DbStatus::SUCCESS;
    }
    /// read objects from the persistent store
    DbStatus read()  {
      return DbStatus::SUCCESS;
    }
  };
}

pool::TestRunner<pool::RootSizePerformance> s_testRootSizePerformance("RootSizePerformance");
