#ifndef ATTRIBUTEIMPLFACTORY_H
#define ATTRIBUTEIMPLFACTORY_H

/// @file 
/// @author Jakub.Moscicki@cern.ch (kuba)

# include "AttributeTypes.h"
# include "Attribute.h"

namespace pool {

  class BaseAttributeFactory
  {
  public:
    virtual ~BaseAttributeFactory() {}
    
    virtual AttributeBaseValue* createAttributeImpl() = 0;
    virtual size_t sizeOfAttributeObject() = 0;
    virtual const std::type_info* type() const = 0;
  };

  template<class T>
  class TypedAttributeFactory : public BaseAttributeFactory
  {
  public:
    TypedAttributeFactory() { AttributeImplFactory::theFactory()._addref(*this, AttributeStaticTypeInfo<T>::type_name()); }

    // performance opt: placement new may be used. two possiblities:
    //  /1/ Attribute lists are read into a cache with specified number of rows (row_number). 
    //      Attributes may be kept in columns, so the allocator would allocate an array [row_number] for each column.
    //      Very size and speed efficient but works only with a cache.
    //  /2/ Because max(sizeOfAttributeObject) is available (for registered types), total sizeof any attribute list (row) to be allocated
    //      is max(sizeOfAttributeObject)*number_of_attributes.
    //      With plain types only, this is equivalent to current (S.Eckman) implementation with discriminated union.
    //      If large complex types could be used as attributes, this solution could be very space innefficient.
    //
    AttributeBaseValue *createAttributeImpl() { return new AttributeTValue<T>(); }
    size_t sizeOfAttributeObject() { return sizeof(AttributeTValue<T>); }
    const std::type_info* type() const { return &( typeid(T) ); }
  };
}

# endif
