#ifndef ATTRIBUTESPECIFICATION_H
#define ATTRIBUTESPECIFICATION_H

/// @file 
/// @author Jakub.Moscicki@cern.ch (kuba)

#include <string>
#include <typeinfo>
#include <iosfwd>

namespace pool {
  /**
   * Specifies type and name of an attribute.
   */
  class AttributeSpecification
  {
  public:
    /// Empty constructor
    AttributeSpecification();

    /// Destructor
    ~AttributeSpecification();

    /// Construct the type from a type_info.
    /// Validity check of the type done at run time.
    AttributeSpecification( std::string name,
                            const std::type_info& typeInfo );
    
    /// Construct the type from a string type name.
    /// Validity check of the type done at run time.
    AttributeSpecification( std::string name,
                            std::string type_name );
    
    /// Construct the type from a static type T.
    /// Validity check of the type done at run time.
    template <class T>
    AttributeSpecification( std::string name );
    
    /// Compare type and name.
    bool operator == (const AttributeSpecification& rhs ) const;
    bool operator != (const AttributeSpecification& rhs ) const;

    /// Attribute name.
    std::string name() const;

    /// Attribute type (as string).
    std::string type_name() const;

    /// Attribute type info
    const std::type_info& type() const;

    void print(std::ostream&) const;

  private:
    /// The attribute name
    std::string m_name;
    /// The attribute type
    const std::type_info* m_type_info;
  };
}

#include "AttributeList/AttributeSpecification.inl"

#endif
