# include "AttributeList/AttributeImplFactory.h"

/// --------- GENERIC TYPE NAMES --------------

namespace pool {

template<> typename_string_t AttributeStaticTypeInfo<bool>::type_name() { return "bool"; }

template<> typename_string_t AttributeStaticTypeInfo<char>::type_name() { return "char"; }

template<> typename_string_t AttributeStaticTypeInfo<unsigned char>::type_name() { return "unsigned char"; }

template<> typename_string_t AttributeStaticTypeInfo<short>::type_name() { return "short"; }

template<> typename_string_t AttributeStaticTypeInfo<unsigned short>::type_name() { return "unsigned short"; }

template<> typename_string_t AttributeStaticTypeInfo<int>::type_name() { return "int"; }

template<> typename_string_t AttributeStaticTypeInfo<unsigned int>::type_name() { return "unsigned int"; }

template<> typename_string_t AttributeStaticTypeInfo<long>::type_name() { return "long"; }

template<> typename_string_t AttributeStaticTypeInfo<unsigned long>::type_name() { return "unsigned long"; }

template<> typename_string_t AttributeStaticTypeInfo<long long>::type_name() { return "long long"; }

template<> typename_string_t AttributeStaticTypeInfo<unsigned long long>::type_name() { return "unsigned long long"; }


template<> typename_string_t AttributeStaticTypeInfo<float>::type_name() { return "float"; }

template<> typename_string_t AttributeStaticTypeInfo<double>::type_name() { return "double"; }

template<> typename_string_t AttributeStaticTypeInfo<long double>::type_name() { return "long double"; }

template<> typename_string_t AttributeStaticTypeInfo<std::string>::type_name() { return "string"; }

template<> typename_string_t AttributeStaticTypeInfo<Token>::type_name() { return "Token"; }


static TypedAttributeFactory<bool> bool_attr_factory;

static TypedAttributeFactory<char> char_attr_factory;
static TypedAttributeFactory<unsigned char> unsigned_char_attr_factory;

static TypedAttributeFactory<short> short_attr_factory;
static TypedAttributeFactory<unsigned short> unsigned_short_attr_factory;

static TypedAttributeFactory<int> int_attr_factory;
static TypedAttributeFactory<unsigned int> unsigned_int_attr_factory;

static TypedAttributeFactory<long> long_attr_factory;
static TypedAttributeFactory<unsigned long> unsigned_long_attr_factory;

static TypedAttributeFactory<long long> long_long_attr_factory;
static TypedAttributeFactory<unsigned long long> unsigned_long_long_attr_factory;

static TypedAttributeFactory<float> float_attr_factory;

static TypedAttributeFactory<double> double_attr_factory;
static TypedAttributeFactory<long double> long_double_attr_factory;

static TypedAttributeFactory<std::string> string_attr_factory;
static TypedAttributeFactory<Token> token_attr_factory;

} /* namespace pool */

/// ------------- RUNTIME MAPPINGS

//// ------------ FAST CONVERTERS -------------

#include <cstdio>
#include <cstdlib>

namespace pool
{

void str_to_mem_bool(const char *c, void *mem)
{
  bool *m = reinterpret_cast<bool*>(mem);
  *m = (std::atoi(c) != 0);
}


void str_to_mem_char(const char *c, void *mem)
{
  char *m = reinterpret_cast<char*>(mem);
  *m = *c;
}
void str_to_mem_unsigned_char(const char *c, void *mem)
{
  unsigned char *m = reinterpret_cast<unsigned char*>(mem);
  *m = std::atoi(c);
}


void str_to_mem_short(const char *c, void *mem)
{
  short *m = reinterpret_cast<short*>(mem);
  *m = std::atoi(c);
}
void str_to_mem_unsigned_short(const char *c, void *mem)
{
  unsigned short *m = reinterpret_cast<unsigned short*>(mem);
  *m = std::atoi(c);
}


void str_to_mem_int(const char *c, void *mem)
{
  int *m = reinterpret_cast<int*>(mem);
  *m = std::atoi(c);
}
void str_to_mem_unsigned_int(const char *c, void *mem)
{
  unsigned int *m = reinterpret_cast<unsigned int*>(mem);
  *m = std::atoi(c); // will it do the proper cast?
}


void str_to_mem_long(const char *c, void *mem)
{
  long *m = reinterpret_cast<long*>(mem);
  *m = std::atol(c);
}

void str_to_mem_unsigned_long(const char *c, void *mem)
{
  unsigned long *m = reinterpret_cast<unsigned long*>(mem);
  *m = std::atol(c);
}


#include <sstream>
void str_to_mem_long_long(const char *c, void *mem)
{
  long long* m = reinterpret_cast<long long*>(mem);
  std::istringstream is(c);
  is >> *m;
}

void str_to_mem_unsigned_long_long(const char *c, void *mem)
{
  unsigned long long* m = reinterpret_cast<unsigned long long*>(mem);
  std::istringstream is(c);
  is >> *m;
}

void str_to_mem_float(const char *c, void *mem)
{
  float *m = reinterpret_cast<float*>(mem);
  *m = float(std::atof(c));
}


void str_to_mem_double(const char *c, void *mem)
{
  double *m = reinterpret_cast<double*>(mem);
  *m = std::strtod(c,0);
}

void str_to_mem_long_double(const char *c, void *mem)
{
  long double *m = reinterpret_cast<long double*>(mem);
  std::istringstream is(c);
  is >> *m;
}

void str_to_mem_str(const char *c, void *mem)
{
  std::string *m = reinterpret_cast<std::string*>(mem);
  *m = (char *)c;
}

void str_to_mem_Token(const char *c, void *mem)
{
  Token *m = reinterpret_cast<Token *>(mem);  
  m->fromString(c);
}


template<class T>
std::pair<typename_string_t, str_to_mem_function> str_mem_converter_map_helper(str_to_mem_function fun)
{
  return std::make_pair(AttributeStaticTypeInfo<T>::type_name(),fun);
}

str_to_mem_function get_str_mem_converter(const typename_string_t& tn)
{
  //singleton
  static std::map<typename_string_t, str_to_mem_function> mmap;

  if(mmap.empty()) // initialize once at the beginning
    {
      mmap.insert(str_mem_converter_map_helper<bool>(str_to_mem_bool));
      mmap.insert(str_mem_converter_map_helper<char>(str_to_mem_char));
      mmap.insert(str_mem_converter_map_helper<unsigned char>(str_to_mem_unsigned_char));
      mmap.insert(str_mem_converter_map_helper<short>(str_to_mem_short));
      mmap.insert(str_mem_converter_map_helper<unsigned short>(str_to_mem_unsigned_short));
      mmap.insert(str_mem_converter_map_helper<int>(str_to_mem_int));
      mmap.insert(str_mem_converter_map_helper<unsigned int>(str_to_mem_unsigned_int));
      mmap.insert(str_mem_converter_map_helper<long>(str_to_mem_long));
      mmap.insert(str_mem_converter_map_helper<unsigned long>(str_to_mem_unsigned_long));
      mmap.insert(str_mem_converter_map_helper<long long>(str_to_mem_long_long));
      mmap.insert(str_mem_converter_map_helper<unsigned long long>(str_to_mem_unsigned_long_long));
      mmap.insert(str_mem_converter_map_helper<float>(str_to_mem_float));
      mmap.insert(str_mem_converter_map_helper<double>(str_to_mem_double));
      mmap.insert(str_mem_converter_map_helper<long double>(str_to_mem_long_double));
      mmap.insert(str_mem_converter_map_helper<std::string>(str_to_mem_str));
      mmap.insert(str_mem_converter_map_helper<Token>(str_to_mem_Token));
    }

  return mmap[tn];
}

}
