#include "CacheEntryImp.h" 
#include "POOLCore/Token.h"  

void pool::CacheEntry::_destroyContent(){
  if ( m_token )   {
    const_cast<Token*>(m_token)->release();
    m_token = 0;
  }
  m_objPtr.reset();
}  

pool::CacheEntry::CacheEntry(): 
          m_objPtr(),m_token(0)  {}
   
pool::CacheEntry::CacheEntry(bool deleteObject): 
          m_objPtr(deleteObject),m_token(0) {}   
 
pool::CacheEntry::CacheEntry(const Token& aToken, bool deleteObject): 
          m_objPtr(deleteObject),m_token(&aToken) {                          
  if ( m_token ) const_cast<Token*>(m_token)->addRef();
}

pool::CacheEntry::CacheEntry(const SharedAnyPtr& aPtr, bool deleteObject): 
            m_objPtr(deleteObject),m_token(0) {
  m_objPtr.copy(aPtr);
}

pool::CacheEntry::CacheEntry(const CacheEntry& aCacheEntry):
           m_objPtr(aCacheEntry.m_objPtr),
           m_token(aCacheEntry.m_token) {}

pool::CacheEntry::~CacheEntry() {
  _destroyContent();
}

pool::CacheEntry& pool::CacheEntry::operator=(const CacheEntry& aCacheEntry){
  m_objPtr = aCacheEntry.m_objPtr;
  m_token = aCacheEntry.m_token;
  (const_cast<Token*>(m_token))->addRef();
  return *this;
}

pool::CacheEntry* pool::CacheEntry::cloneObjectContent(){
  CacheEntry* loc = new CacheEntry(m_objPtr,m_objPtr.deletePolicy());
  return loc;
}

bool pool::CacheEntry::setToken(const Token* aToken){
  bool ret = false;
  //if(!m_token && aToken && m_objPtr.get()){
  if(!m_token && aToken){
    m_token = aToken;
    const_cast<Token*>(m_token)->addRef();
    ret = true;
  //} else if(!aToken && m_objPtr.get()){
  } else if(!aToken){
    if(m_token){
      const_cast<Token*>(m_token)->release();
    }
    m_token = 0;
  }
  return ret;
}

const pool::Token* pool::CacheEntry::token() const {
  return m_token;
}

bool pool::CacheEntry::setObject(const SharedAnyPtr& aPtr){
  bool ret = false;
  if(aPtr.get()){
    m_objPtr.copy(aPtr);
    ret = true;
  } 
  return ret;        
}

const pool::SharedAnyPtr& pool::CacheEntry::object() const {
  return m_objPtr;
}

