// 
#ifndef POOL_CACHEENTRYIMP_H
#define POOL_CACHEENTRYIMP_H 

#include "DataSvc/CacheEntry.h"
#include "RefCounter.h"

namespace pool    {

  /** @class CacheEntryImp CacheEntryImp.h
      Unit of storage memory for the DataSvc cache. A SINGLE cache line is created 
      for both input object (defined by the object pointer) and for output object (defined by the
      storage token). Life of transient object read form DB depends directly on the life of the 
      related cache line.
      @author Giacomo Govi
      @date 10/2/2003
  */
  class CacheEntryImp : public RefCounter, public CacheEntry {

    public:
  
    // default ctor
      CacheEntryImp();

      // default ctor
      CacheEntryImp(bool deleteObject);

      // Token ctor
      CacheEntryImp(const Token& aToken, bool deleteObject=true);

      //                                      
      CacheEntryImp(const SharedAnyPtr& aPtr, bool deleteObject=true);


      CacheEntryImp(const CacheEntryImp& aCacheEntry);

      // standard destructor
      virtual ~CacheEntryImp();

      CacheEntryImp& operator=(const CacheEntryImp& aCacheLine);

      CacheEntry* cloneObjectContent();

      // reset the fields describing the object
      bool resetObject();

      // reset the fields describing the object
      bool deleteObject();

      // reset all fields of the cache line but the reference counter
      void reset();

      // set the cache line content to be marked for write. After the commit the object pointer
      // will be reset
      void setMarkWrite(bool set_flag=true);

      // verifies if the current object has been marked for write
      bool isMarkedWrite() const;
       
      // set the cache line content to be marked for update. After the commit the object pointer
      // will be reset
      void setMarkUpdate(bool set_flag=true);

      // verifies if the current object has been marked for update
      bool isMarkedUpdate() const;

      // set the cache line content to be marked for delete. After the commit the object pointer
      // will be reset
      void setMarkDelete(bool set_flag=true);

      // verifies if the current object has been marked for delete
      bool isMarkedDelete() const;

      void unMark();

      bool isMarked() const;

    private:

      // flag to mark for write
      bool marked_write;

      bool marked_update;
  
      bool marked_delete;

      // method to destroy the transient object in the cache line
      void _destroyObject();

  };

}

#endif  // POOL_CACHEENTRYIMP_H


