// $Header: /cvs/PF/pool/DataSvc/src/Hmap_kptr_table.h,v 1.9 2003/11/04 08:25:02 govi Exp $
#ifndef POOL_HMAP_KPTR_TABLE_H
#define POOL_HMAP_KPTR_TABLE_H 

#include "Hmap_k_table.h"

namespace pool  {

  /** @struct KeyPtr Hmap_kptr_table.h
      Dummy struct implemented to encapsulate the pointer to be used as a 
      key for the search in the table.
      
      @author Giacomo Govi
      @date 25/07/2002
  **/
  struct KeyPtr {
      
    const void* thePtr;

    // constructor
    KeyPtr(const void* aPtr){
      thePtr = aPtr;
    }

    bool operator==(const KeyPtr &kr)   const {  
      return thePtr == kr.thePtr;       
    }
    bool operator<(const KeyPtr &kr)    const {  
      return thePtr < kr.thePtr;       
    }

    long toLong() const {
       long ts = (long)thePtr;
       ts = ts>>8;
       return ts;
    }

  };

}

namespace EXTSTD {

#ifndef __ICC
   // Standard __gnu_cxx hash mechanism.
   template <> struct hash<pool::KeyPtr>  {
     unsigned int operator()(const pool::KeyPtr& kp) const {
       return hash<long>().operator()(kp.toLong());
     }
   };

   // hash function for win32
   inline unsigned int hash_value(const pool::KeyPtr& kp) { 
     return hash<pool::KeyPtr>().operator()(kp);    
   }

#else

  template <> struct less<pool::KeyPtr> {
    bool operator()(const pool::KeyPtr& k1, const pool::KeyPtr& k2) const {
      return k1<k2;
    }
  };  

  // ICC (dinkumware) hash mechanism.
  template <> class hash_compare<pool::KeyPtr,EXTSTD::less<pool::KeyPtr> > {
    less<pool::KeyPtr> comp;

    public:
    enum { // parameters for hash table
	  bucket_size = 4,	// 0 < bucket_size
	  min_buckets = 8       // min_buckets = 2 ^^ N, 0 < N
    };	

    hash_compare() :comp(){}

    hash_compare(less<pool::KeyPtr> pred): comp(pred){}

    size_t operator()(const pool::KeyPtr& kp) const {
      return hash_compare<long>().operator()(kp.toLong());
    }

    bool operator()(const pool::KeyPtr& key1, const pool::KeyPtr& key2) const {
      return comp(key1,key2);
    }
  };
#endif

}

namespace pool  {

  /** @class Hmap_kptr_table Hmap_kptr_table.h
      Templated container class. Designed to be a look up table to map a set of pointers
      to an other set. The implemetnation uses an hash_map<KeyPtr,void*> where the KeyPtr is
      a structure holding the pointer used as akey.

      @author Giacomo Govi
      @date 25/07/2002
   **/
  template <class T> class Hmap_kptr_table : public Hmap_k_table<KeyPtr,T> {    

    public:

      // default constructor
      Hmap_kptr_table() : Hmap_k_table<KeyPtr,T>() {}

      // constructor with initial capacity
      Hmap_kptr_table(unsigned int init_size) : Hmap_k_table<KeyPtr,T>(init_size){}

  };  

}

#endif  // POOL_HMAP_KPTR_TABLE_H








