#include "ReferenceHandler.h"
#include "DataSvc/ICacheSvc.h"
#include "DataSvc/RefBase.h"

/// Standard Constructor
pool::ReferenceHandler::ReferenceHandler(ICacheSvc* cache)
: m_cache(cache)
{
}

/// Standard Destructor
pool::ReferenceHandler::~ReferenceHandler() 
{
}

/// Convert transient reference to persistent OID
pool::DbStatus
pool::ReferenceHandler::saveReference(Reference* pRef, Token::OID_t* data)
{
  RefBase* r = dynamic_cast<RefBase*>(pRef);
  if(!r){
    data->first = data->second = 0xFFFFFFFF;
    return pool::DbStatus::Error;
  }
  if(!r->cacheSvc()){
    r->setCacheSvc(m_cache);
  }
  const Token* tok = r->token();
  if ( !tok )  {
    // Serious error, but unrecoverable; unclear what to do
    // This reference can never ever be followed when reading.
    data->first = data->second = 0xFFFFFFFF;
    return pool::DbStatus::Error;
  }
  return saveToken(tok, data);
}

/// Convert persistent OID into a transient reference
pool::DbStatus
pool::ReferenceHandler::loadReference(Reference* pRef, Token::OID_t* data)
{
  RefBase* r = dynamic_cast<RefBase*>(pRef);
  if(!r){
    return pool::DbStatus::Error;
  }
  Token *tok = 0;
  if(!r->cacheSvc()){
    r->setCacheSvc(m_cache);
  }
  pool::DbStatus sc = createToken(data, tok);
  if ( sc.isSuccess() )   {
    if(tok){
      r->setToken(*tok);
      tok->release();
    }
  }
  return sc;
}

