#define POOL_SMARTREFTEST_CPP 1

#include <iostream>
#include "DataSvc/Ref.h"
#include "DataSvc/RefException.h"
#include "src/CacheEntryImp.h"
#include "src/DataSvc.h"
#include "src/CacheSvc.h"
#include "../Common/FakeToken.h"
#include "../Common/TestDataObject.h"
#include "../Common/DummyPersistencySvc.h"
#include "../Common/TestKernel.h"

using namespace pool;

class RefTest : public TestKernel {
  
  public:
  typedef TestDataObject T;

  RefTest() : TestKernel(){
    theDataSvc =  new DataSvc(new DummyPersistencySvc);
    theCacheSvc = &theDataSvc->cacheSvc();
  }

  virtual ~RefTest(){
    delete theDataSvc;
  }

  void _dumpObjects(){
    TestDataObject::dumpObjects();
  }

  bool basicTest(Ref<T>& r){
    bool ret = true;
    try{
      r->doJob();
    } catch (pool::Exception e){
      printE(e);
      ret = false;
    }
    printM("Arrow operator test completed\n");
    return ret;
  }

  void operatorTest(Ref<T>& r){
    long id1=0,id2=0,id3=0,id4=0;
    try{
      id1 = r->getOId();
    } catch (pool::Exception e){
      printE(e);
      printM("Arrow operator failed.\n");
    }
    try{
      id2 = (*r).getOId();
    } catch (pool::Exception e){
      printE(e);
      printM("Star operator failed.\n");
    }
    try{
      T* t = r;
      id3 = t->getOId();
    } catch (pool::Exception e){
      printE(e);
      printM("Conversion operator failed.\n");
    }
    try{
      T* t = r.ptr();
      id4 = t->getOId();
    } catch (pool::Exception e){
      printE(e);
      printM("ptr() method failed.\n");
    }
    if(id1 != id2){
      std::cout<<"test ERROR: from '->' got="<<id1<<" from '*' got="<<id2<<"\n";
    }
    if(id2 != id3){
      std::cout<<"test ERROR: from '*' got="<<id2<<" from conv got="<<id3<<"\n";
    }
    if(id3 != id4){
      std::cout<<"test ERROR: from conv got="<<id3<<" from ptr() got="<<id4<<"\n";
    }
    printM("Access Operators test completed\n");
  }

  bool ptrcheckTest(Ref<T>& r,bool expected){
    bool ret = true;
    bool bp = false;
    try{
      bp = r != 0;
    } catch (pool::Exception e){
      ret = false;
      printE(e);
      printM("Exception: Conv failed.\n");
    }
    bool bn = false;
    try{
      bn = !r;
    } catch (pool::Exception e){
      ret = false;
      printE(e);
      printM("Exception: Not failed.\n");
    }
    if(expected != bp){
      printM("Result of conv wrong\n");
      ret = false;
    }
    if(expected == bn){
      printM("Result of not wrong\n");  
      ret = false;
    }
    printM("Ptr check test completed\n");
    return ret;
  }   

  void equalsOperatorTest(Ref<T>& r1,Ref<T>& r2,bool expected){
    bool r1r2 = (r1==r2);
    bool r2r1 = (r2==r1);
    if(r1r2 != expected)printError("Test ERROR: Result is different from expected\n");
    if(r1r2 != r2r1)printError("Test ERROR: Backward result is different \n");
    r1r2 = (r1!=r2);
    r2r1 = (r2!=r1);
    if(r1r2 == expected)printError("Test ERROR: 'Not' Result is different from expected\n");
    if(r1r2 != r2r1)printError("Test ERROR: Backward result is different \n");
    printM("Equality ref-ref operator test completed\n");
  }

  void equalsOperatorTest(Ref<T>& r1,const T* t2,bool expected){
    bool r1t2 = (r1==t2);
    bool t2r1 = (t2==r1);
    if(r1t2 != expected)printError("Test ERROR: Result is different from expected\n");
    if(r1t2 != t2r1)printError("Test ERROR: Backward result is different \n");
    r1t2 = (r1!=t2);
    t2r1 = (t2!=r1);
    if(r1t2 == expected)printError("Test ERROR: 'Not' Result is different from expected\n");
    if(r1t2 != t2r1)printError("Test ERROR: Backward result is different \n");
    printM("Equality ptr-ref operator test completed\n");
  }

  void _fullTest(bool full,Ref<T>& r){
    if(full) operatorTest(r);
    //Token* t = new FakeToken(1);
    //Ref<T> r2(theDataSvc,*t);
    Ref<T>* r2=0;
    try{
      r2 =  new Ref<T>(theDataSvc,new T(1));
      r2->ptr();
    } catch (pool::Exception e){
      printE(e);      
    }
    //equalsOperatorTest(r,*r2,false);
    //T* o = *r2;
    //equalsOperatorTest(r,o,false);
    //r = o;
    //equalsOperatorTest(r,*r2,true);
    //equalsOperatorTest(r,o,true);
    //delete r2;        
    printM("Full test completed");
  }

  void _emptyConstrTest(){
    printM("Testing empty Ref - no DataSvc, no ptr *******\n");
    Ref<T> r;
    if(basicTest(r)){
      printError("test ERROR: basic test failed\n");
    }
    if(!ptrcheckTest(r,false)){
      printError("test ERROR: ptrcheck failed\n");
    }    
    _fullTest(false,r);
  }    

  void _DSvcConstrTest(){
    printM("Testing empty Ref - only DataSvc, no ptr *******\n");
    Ref<T> r(theDataSvc);
    if(basicTest(r)){
      printError("test ERROR: basic test failed\n");
    }
    if(!ptrcheckTest(r,false)){
      printError("test ERROR: ptrcheck failed\n");
    }    
    _fullTest(false,r);
  }    

  void _ptrConstrTest(){
    printM("Testing Ref - DataSvc + ptr *******\n");
    T* obj = new T(299);
    try{
      Ref<T> r(theDataSvc,obj);
      //if(!basicTest(r)){
      if(basicTest(r)){
        //printError("test ERROR: exception has been thrown\n");
        printError("test ERROR: exception has not been thrown\n");
      }
      //if(!ptrcheckTest(r,true)){
      //  printError("test ERROR: exception has been thrown\n");
      //}    
      //_fullTest(true,r);
    } catch (pool::Exception e){
      printE(e);
    } 
  }    

  void _tokConstrTest(){
    printM("Testing Ref - DataSvc + token *******\n");
    Token* tok = new FakeToken(99);
    try{
      Ref<T> ref(theDataSvc,*tok);
      Token* t = const_cast<Token*>(ref.token());
      Ref<T> r(theDataSvc,*t);
      if(basicTest(r)){
	//if(!basicTest(r)){
        //printError("test ERROR: exception has been thrown\n");
        printError("test ERROR: exception has not been thrown\n");
      }
      //if(!ptrcheckTest(r,true)){
      //  printError("test ERROR: exception has been thrown\n");
      //}    
      //_fullTest(true,r);
    } catch (pool::Exception e){
      printE(e);
    } 
  }    

  void _stokConstrTest(){
    printM("Testing Ref - DataSvc + string token *******\n");
    Token* t = new FakeToken(99);
    try{
      Ref<T> ref(theDataSvc,*t);
      const std::string& s = ref.toString();
      Ref<T> r(theDataSvc,s);    
      //if(!basicTest(r)){
      if(basicTest(r)){
        //printError("test ERROR: exception has been thrown\n");
        printError("test ERROR: exception has not been thrown\n");
      }
      //if(!ptrcheckTest(r,true)){
      //  printError("test ERROR: exception has been thrown\n");
      //}    
      //_fullTest(true,r);
    } catch (pool::Exception e){
      printE(e);
    } 
  }    

  void _refConstrTest(){
    printM("Testing Ref(ref) ******* \n");
    try{
      Ref<T> ref(theDataSvc,new T(99));
      T* ptr = ref;
      Ref<T> r(ref);
      //if(!basicTest(r)){
      if(basicTest(r)){
        //printError("test ERROR: exception has been thrown\n");
        printError("test ERROR: exception has not been thrown\n");
      }
      //if(!ptrcheckTest(r,true)){
      //  printError("test ERROR: exception has been thrown\n");
      //}    
      equalsOperatorTest(r,ref,true);
      equalsOperatorTest(r,ptr,true);
      //Ref<T> anOther(theDataSvc,*(new FakeToken(199)));
      Ref<T> anOther(theDataSvc,new T(199));
      //T* anOtherPtr = anOther;
      equalsOperatorTest(r,anOther,false);
      //equalsOperatorTest(r,anOtherPtr,false);
      //_fullTest(true,r);
    } catch (pool::Exception e){
      printE(e);
    }     
  }    

  void _ptrAssignTest(){
    printM("Testing Ref=ptr ******* \n");
    try{
      Ref<T> ref(theDataSvc,new T(99));
      T* ptr = ref;
      Ref<T> r(theDataSvc);
      r = ptr;
      //if(!basicTest(r)){
      if(basicTest(r)){
        //printError("test ERROR: exception has been thrown\n");
        printError("test ERROR: exception has not been thrown\n");
      }
      //if(!ptrcheckTest(r,true)){
      //  printError("test ERROR: exception has been thrown\n");
      //}    
      //equalsOperatorTest(r,ref,true);
      //equalsOperatorTest(r,ptr,true);
      //Ref<T> anOther(theDataSvc,new T(199));
      //T* anOtherPtr = anOther;
      //equalsOperatorTest(r,anOther,false);
      //equalsOperatorTest(r,anOtherPtr,false);
      //_fullTest(true,r);
    } catch (pool::Exception e){
      printE(e);
    } 
  }   

   void _refAssignTest(){
    printM("Testing Ref=ref *******\n");
    try{
      Ref<T> ref(theDataSvc,new T(99));
      //T* ptr = ref;
      Ref<T> r;
      r = ref;
      //if(!basicTest(r)){
      if(basicTest(r)){
        //printError("test ERROR: exception has been thrown\n");
        printError("test ERROR: exception has not been thrown\n");
      }
      //if(!ptrcheckTest(r,true)){
      //  printError("test ERROR: exception has been thrown\n");
      //}    
      equalsOperatorTest(r,ref,true);
      //equalsOperatorTest(r,ptr,true);
      Ref<T> anOther(theDataSvc,new T(199));
      //T* anOtherPtr = anOther;
      equalsOperatorTest(r,anOther,false);
      //equalsOperatorTest(r,anOtherPtr,false);
      //_fullTest(true,r);
    } catch (pool::Exception e){
      printE(e);
    } 
  }  
  
  class DT : public T {
    public:
    DT():T(0){}
    DT(long l):T(l){}
    void run(){};
  };

  void _SintaxOK(){
    printM("Testing Inheritance *******\n");
    try{
      long OID = 5;
      DT* o = new DT(OID);
      _dumpObjects();
      Ref<T> r(theDataSvc);
      r = o;
      Ref<T> r2(theDataSvc);
      Ref<DT> rd(theDataSvc);
      rd = o;
      r2 = rd;
      long v = r->getOId();
      if(v!=r2->getOId()){
        printError("test ERROR: assignement from a extended class failed");
      }
      if(v!=rd->getOId()){
        printError("test ERROR: assignment of a pointer to a ref extended failed");
      }
      Ref<T> r3(theDataSvc,rd);
      T* p = rd;
      if(p->getOId() != OID){
        printError("test ERROR!\n");
      }
      bool eq = false;
      _fullTest(true,r);
      _fullTest(true,r2);
      eq = (rd == 0);
      if(eq){
        printError("test ERROR!rd != 0!");
      }
      _dumpObjects();
    } catch (pool::Exception e){
      printE(e);
    }     
  }

  void refScope(Ref<T>& r){
    CacheEntryImp* e = 0;
    const Token *tok =  r.token();
    if(tok){
      e = dynamic_cast<CacheEntryImp*>(theCacheSvc->registerToken(*tok));
    } 
    // here the copy constructor is called
    Ref<T> r2 = r;
    std::cout<< "S Refs ="<<e->refCount()<<"\n"; 
  }

  void refCreate(Token& t){
    try{
      Ref<T> r0(theDataSvc,t);
      CacheEntryImp* en = dynamic_cast<CacheEntryImp*>(theCacheSvc->registerToken(t));
      if(en){
        std::cout <<"1 Refs ="<<en->refCount()<<"\n";
        refScope(r0);
        std::cout<< "2 Refs ="<<en->refCount()<<"\n";
        Ref<T> r1(r0);
        std::cout <<"3 Refs ="<<en->refCount()<<"\n";
        Ref<T> r2;
        r2 = r1;
        std::cout <<"4 Refs ="<<en->refCount()<<"\n";
        Ref<T> r3(theDataSvc);
        r3 = r2.ptr();
        std::cout <<"5 Refs ="<<en->refCount()<<"\n";
      } else {
        printError("test ERROR: cannot resolve cache line");
      }
    } catch (pool::Exception exc){
      printE(exc);
    }     
  }

  void _refCountingTest(){
    printM("Testing ref counting *******");
    Token* t = new FakeToken(999999);
    try{
      Ref<T> r(theDataSvc,*t);
      refCreate(*t);
      CacheEntryImp* en= dynamic_cast<CacheEntryImp*>(theCacheSvc->registerToken(*t));
      if(en){
        std::cout <<"M Refs ="<<en->refCount()<<"\n";
      }
    } catch (pool::Exception exc){
      printE(exc);
    }     
  }

  void _convTest(const T* arg){
    if(arg)
    arg->doJob();
  }

  void _convTestNC(T* arg){
    if(arg)
    arg->doJob();
  }

  void _constnessTest(){
    printM("Testing ref costness *******");
    try{
      const DT* o0 = new DT;
      Ref<const DT> r0(theDataSvc);
      r0 = o0;
      //o0->run();  // 
      //r0->run();  // should not compile!!
      DT* o1 = new DT;
      Ref<const DT> r1;
      r1 = o1;
      r0 = o1;
      r0 = r1;
      o1->run();  // 
      r1 = o0;
      _convTest(r0);
      Ref<const DT> r3(theDataSvc,o1);
      r3 = o0;
      const Ref<const DT> r4(theDataSvc,o1);
      r3 = r4;
      r3 = o1;
      Ref<DT> r5;
      r5 = o1;
      r3 = r5;
    } catch (pool::Exception e){
      printE(e);
    }     
  }

  void _conversionTest(){
    printM("Testing ref conversion *******");
    try{
      Ref<T> r0;
      Ref<DT> r1(theDataSvc,new DT);
      r0 = r1;
      if(r0->getOId()!=r1->getOId()){
        printError("test ERROR: r0!=r1");
      }
    } catch (pool::Exception e){
      printE(e);
    }     
  }

  void _ptrScope(){
    T* t0 = new T(100);
    T* t1 = new T(101);
    try{
      Ref<T> r3(theDataSvc,new T(102));
      Ref<T> r4;
      r4 = r3.ptr();
      _dumpObjects();
      Ref<T> r0;
      r0 = t0;
      Ref<T> r1(theDataSvc);
      r1 = t0;
      r0 = t1;
      _dumpObjects();
    } catch (pool::Exception e){
      printE(e);
    }     
  }


  void _ptrOwnerShipTest(){
    printM("Testing ptr ownership *******");
    _ptrScope();
    _dumpObjects();
  }


  private:
    IDataSvc* theDataSvc;  
    ICacheSvc* theCacheSvc;

};

int main(int, char **)
{
  RefTest test;
  test._emptyConstrTest(); 
  test._DSvcConstrTest();
  test._ptrConstrTest();
  test._refConstrTest();
  test._ptrAssignTest();
  test._refAssignTest();
  test._SintaxOK();
  test._constnessTest();
  test._conversionTest();
  test._ptrOwnerShipTest();
  test._dumpObjects();
  test.dumpError();
  return 0; 
}
