#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALDATABASEDESCRIPTOR_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALDATABASEDESCRIPTOR_H

#include <string>
#include <vector>
#include <set>

namespace pool {

  class ObjectRelationalMappingDescriptor;

  class ObjectRelationalClassDescriptor {
  public:

    ObjectRelationalClassDescriptor();

    ObjectRelationalClassDescriptor(const std::string className,
                                    const std::string mappingVersions);

    virtual ~ObjectRelationalClassDescriptor();

    const std::string& className() const;

    const std::string& mappingVersion() const;

    const ObjectRelationalMappingDescriptor* mapping() const;
    
    ObjectRelationalMappingDescriptor* newMapping();
    
    private:

    std::string m_className;

    std::string m_mappingVersion;

    ObjectRelationalMappingDescriptor* m_mapping;

  };

  class ObjectRelationalContainerDescriptor {
  public:

    ObjectRelationalContainerDescriptor();

    ObjectRelationalContainerDescriptor(const std::string& name, const std::string& type);

    virtual ~ObjectRelationalContainerDescriptor();

    const std::string& name() const;

    const std::string& type() const;

    ObjectRelationalClassDescriptor* addClassStore(const std::string& className, const std::string& mappingVersion);

    const std::vector<ObjectRelationalClassDescriptor*>& classStores() const;

    void setRecreatePolicy(bool flag);

    bool recreatePolicy() const;

    static std::string homogeneousContainerTypeName();
    
    static std::string polymorphicContainerTypeName();

    private:

    std::string m_name;

    std::string m_type;

    std::vector<ObjectRelationalClassDescriptor*> m_classStores;

    bool m_recreate;

  };

  class ObjectRelationalDatabaseDescriptor {
  public:

    ObjectRelationalDatabaseDescriptor();

    virtual ~ObjectRelationalDatabaseDescriptor();

    ObjectRelationalContainerDescriptor& addContainer(const std::string& containerName,const std::string& containerType );

    const std::vector<ObjectRelationalContainerDescriptor*>& containers() const;

    std::set<std::string> involvedClasses() const;

    private:

    std::vector<ObjectRelationalContainerDescriptor*> m_containers;

  };

}

#endif // OBJECTRELATIONALACCESS_OBJECTRELATIONALDATABASEDESCRIPTOR_H
