#ifndef OBJECTRELATIONALACCESS_MAPPINGBUILDER_H
#define OBJECTRELATIONALACCESS_MAPPINGBUILDER_H

#include <string>
#include <set>
#include <vector>
#include <map>
#include "ObjectRelationalAccess/ObjectRelationalMappingPersistency.h"

namespace coral {
  class ISchema;
}

namespace pool {
  
  // forward declarations
  class ObjectRelationalMappingDescriptor;
  class ObjectRelationalMapping;

  class MappingBuilder {

  public:

    MappingBuilder( coral::ISchema& schema);

    virtual ~MappingBuilder();

    void buildMapping(const ObjectRelationalMappingDescriptor* descriptor);

    bool exportMapping(const std::string& fileName);

    ObjectRelationalMapping* retrieveMapping(const std::string& version);

    bool retrieveMapping(const std::string& version,const std::string& fileName);

    bool retrieveMapping(const std::string& fileName, bool oldVersionSchema);

    bool retrieveAllContainerMappings(std::ostream& os, const std::string& containerName);

    bool retrieveContainerMapping(std::ostream& os, const std::string& containerName, std::string classVersion=std::string(""));

    std::map<std::pair<std::string,std::string>, std::string > mappingForClassVersions();
    
    std::map<std::string,std::string> mappingVersions();

    std::map<std::string, std::string> containerTableMap();

    void selectMapping(const std::string& version, std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& mappingStruct);

    void finalizeMapping(bool evolve, bool dryRun);

    bool removeMapping( const std::string& version, bool removeTables=false, bool dryRun=false);

    bool upgradeMappingSchema();

    bool storeMapping( const std::string& containerName, const ObjectRelationalMapping& mapping, std::string classVersion, bool materialize);

    std::string classVersion( const std::string& containerName );

    private:
    
    typedef std::pair<std::string,std::string> ClassMetadata;

    void init(bool forWriting=true);

    void deleteMappings();

    bool m_initialized;

    std::map<std::string,std::pair<ObjectRelationalMapping*,ClassMetadata> > m_builtMappings;

    std::map<std::string,std::pair<std::string,std::string> > m_containerMappings;

    std::map<std::string, bool> m_mappingVersions;

    coral::ISchema& m_schema;

    ObjectRelationalMappingPersistency m_mappingPersistency;

    std::map<std::string,std::string> m_containerTableMap;
    
  };

}

#endif
