#include "ObjectRelationalAccess/ObjectRelationalTableNameRationalizer.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingElement.h"
#include "ObjectRelationalAccess/ObjectRelationalDefaultMappingRules.h"
#include "RelationalAccess/ISchema.h"
#include <sstream>

pool::ObjectRelationalTableNameRationalizer::ObjectRelationalTableNameRationalizer( coral::ISchema& schema ):
  m_schema( schema ),
  m_maximumSizeForTableName( pool::ObjectRelationalTableNameRationalizer::defaultMaximumSizeForTableName ),
  m_validTableNames(),
  m_suggestedToValidName()
{}

pool::ObjectRelationalTableNameRationalizer::ObjectRelationalTableNameRationalizer( coral::ISchema& schema,
                                                                                    unsigned int maximumSizeForTableName ):
  m_schema( schema ),
  m_maximumSizeForTableName( maximumSizeForTableName ),
  m_validTableNames(),
  m_suggestedToValidName()
{}

pool::ObjectRelationalTableNameRationalizer::~ObjectRelationalTableNameRationalizer()
{}

std::string
pool::ObjectRelationalTableNameRationalizer::validNameForNewTable( const std::string& suggestion )
{
  // Check if this is an already validated name
  std::map< std::string, std::string >::const_iterator iValidName = m_suggestedToValidName.find( suggestion );
  if ( iValidName != m_suggestedToValidName.end() ) return iValidName->second;

  // Check name as is first
  if (  suggestion.size() <= m_maximumSizeForTableName &&
        ! m_schema.existsTable( suggestion ) &&
        ! m_schema.existsView( suggestion ) ) {
    m_suggestedToValidName.insert( std::make_pair( suggestion, suggestion ) );
    m_validTableNames.insert( suggestion );
    return suggestion;
  }

  unsigned int i=0;
  std::string candidateName( suggestion );
  while( m_schema.existsTable( candidateName ) || m_schema.existsView( candidateName ) || (m_validTableNames.find( candidateName ) != m_validTableNames.end())){
    candidateName = pool::ObjectRelationalDefaultMappingRules::newNameForSchemaObject( suggestion, i, pool::ObjectRelationalDefaultMappingRules::MaxTableNameLength );
    i++;
  }
  m_suggestedToValidName.insert( std::make_pair( suggestion, candidateName ) );
  m_validTableNames.insert( candidateName );
  return candidateName;
}

void
pool::ObjectRelationalTableNameRationalizer::rationalizeMappingElement( pool::ObjectRelationalMappingElement& element )
{
  element.alterTableName( this->validNameForNewTable( element.tableName() ) );
  for ( pool::ObjectRelationalMappingElement::iterator iElement = element.begin();
        iElement != element.end(); ++iElement ) {
    this->rationalizeMappingElement( iElement->second );
  }
}
