#include "ObjectRelationalAccess/ObjectRelationalTableVariableNameRationalizer.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingElement.h"
#include <sstream>

pool::ObjectRelationalTableVariableNameRationalizer::ObjectRelationalTableVariableNameRationalizer():
  m_maximumSizeForTableVariableName( pool::ObjectRelationalTableVariableNameRationalizer::defaultMaximumSizeForTableVariableName ),
  m_validTableVariableNames(),
  m_suggestedToValidName()
{}


pool::ObjectRelationalTableVariableNameRationalizer::ObjectRelationalTableVariableNameRationalizer( unsigned int maximumSizeForTableVariableName ):
  m_maximumSizeForTableVariableName( maximumSizeForTableVariableName ),
  m_validTableVariableNames(),
  m_suggestedToValidName()
{}


pool::ObjectRelationalTableVariableNameRationalizer::~ObjectRelationalTableVariableNameRationalizer()
{}


std::string
pool::ObjectRelationalTableVariableNameRationalizer::validNameForNewTableVariable( const std::string& tableName,
                                                                                   const std::string& suggestion )
{
  // Check if this is an already validated name
  std::map< std::string, std::string >& suggestedToValidName = m_suggestedToValidName[ tableName ];
  std::map< std::string, std::string >::const_iterator iValidName = suggestedToValidName.find( suggestion );
  if ( iValidName != suggestedToValidName.end() ) return iValidName->second;

  // Check name as is first
  if ( suggestion.size() <= m_maximumSizeForTableVariableName ) {
    suggestedToValidName.insert( std::make_pair( suggestion, suggestion ) );
    m_validTableVariableNames[tableName].insert( suggestion );
    return suggestion;
  }

  std::set< std::string >& validTableVariableNames = m_validTableVariableNames[tableName];
  for ( unsigned int iTrial = 1; static_cast<unsigned int>( iTrial / 10 ) + 1 < m_maximumSizeForTableVariableName; ++iTrial ) {
    // Calculate the prefix
    unsigned int digitsForPrefix = iTrial / 10 + 1;
    unsigned int prefixSize = m_maximumSizeForTableVariableName - digitsForPrefix - 1;
    std::ostringstream os;
    os << suggestion.substr( 0, prefixSize ) << "_" << iTrial;
    std::string candidateName = os.str();
    if ( validTableVariableNames.find( candidateName ) == validTableVariableNames.end() ) {
      suggestedToValidName.insert( std::make_pair( suggestion, candidateName ) );
      validTableVariableNames.insert( candidateName );
      return candidateName;
    }
  }

  return "";
}


void
pool::ObjectRelationalTableVariableNameRationalizer::rationalizeMappingElement( pool::ObjectRelationalMappingElement& element )
{
  const std::string& tableName = element.tableName();
  const std::vector< std::string >& columnNames = element.columnNames();
  std::vector< std::string > newColumnNames;
  newColumnNames.reserve( columnNames.size() );
  for ( std::vector< std::string >::const_iterator iColumnName = columnNames.begin();
        iColumnName != columnNames.end(); ++iColumnName ) {
    newColumnNames.push_back( this->validNameForNewTableVariable( tableName, *iColumnName ) );
  }
  element.setColumnNames( newColumnNames );

  for ( pool::ObjectRelationalMappingElement::iterator iElement = element.begin();
        iElement != element.end(); ++iElement ) {
    this->rationalizeMappingElement( iElement->second );
  }
}
