#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/Exception.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include <iostream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-f";
  opt.property = "file";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the mapping xml file name";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-o";
  opt.property = "outputfile";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the full mapping xml file name (output)";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-e";
  opt.property = "evolve";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "allow to update existing objects for schema evolution";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-dry";
  opt.property = "dryrun";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "only build mapping and dump required changes on the schema";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;  
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::ObjectRelationalMappingUtilities utility;
      bool evolve = false;
      iter = ops.find("evolve");
      if(iter!=ops.end()){
        evolve = true;
      }      
      bool dry = false;
      iter = ops.find("dryrun");
      if(iter!=ops.end()){
        dry = true;
      }      
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                            pool::ObjectRelationalMappingUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                              pool::ObjectRelationalMappingUtilities::infoVerbosityPropertyValue());
        }
      }
      iter = ops.find("file");
      if(iter==ops.end()){
        throw pool::Exception("Mapping file not provided", "", "pool_build_object_relational_mapping" );
      } 
      std::string theMappingFile = (*iter).second;
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      } 
      std::string theConnectionString(""); 
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = (*iter).second;
      }
      if(theConnectionString.empty()){
        throw pool::Exception("Connection string not provided.", "", "pool_build_object_relational_mapping" );
      }
      
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = (*iter).second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),thePassword);
      } 
      std::string theOutputFile;
      iter = ops.find("outputfile");
      if(iter!=ops.end()){
        theOutputFile = (*iter).second;
      } else {
        if(dry) theOutputFile = "mapping.xml";
      }
      utility.initialize();
      utility.connect(theConnectionString,coral::Update);
      utility.session()->transaction().start();
      try {
        utility.buildAndMaterializeMapping(theMappingFile,theOutputFile,evolve,dry);
        if(!dry){
          std::cout << "Mapping built and materialized." << std::endl;
          if(evolve) std::cout << "Required changes applied on the schema." << std::endl;
        } else{
          std::cout << "Mapping built. Output written in file \""<< theOutputFile<<"\"."<<std::endl;
        }
        utility.session()->transaction().commit();
      } catch (const std::exception& e){
        std::cout << "ERROR: " << e.what() << std::endl;
        utility.session()->transaction().rollback();
        return 1;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

