#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/Exception.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include <iostream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-v";
  opt.property = "version";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the version label of the mapping to retrieve";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-o";
  opt.property = "outputfile";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the mapping xml output file name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-n";
  opt.property = "containerName";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the selected container name";
  opt.required = false;
  ops.push_back(opt);  
  opt.flag = "-cv";
  opt.property = "classVersion";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the class version";
  opt.required = false;
  ops.push_back(opt);  
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = false;
  ops.push_back(opt);  
  opt.flag = "-all";
  opt.property = "allVersions";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "retrieve all available versions";
  ops.push_back(opt);   
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::ObjectRelationalMappingUtilities utility;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                            pool::ObjectRelationalMappingUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                              pool::ObjectRelationalMappingUtilities::infoVerbosityPropertyValue());
        }
      }
      std::string theVersion("");
      iter = ops.find("version");
      if(iter!=ops.end()){
        theVersion = iter->second;
      } 
      std::string theMappingOutFile = "mapping.xml";
      iter = ops.find("outputfile");
      if(iter!=ops.end()){
        theMappingOutFile = iter->second;
      } 
        std::string theConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = iter->second;
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = iter->second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),thePassword);
      }
      std::string contName("");
      iter = ops.find("containerName");
      if(iter!=ops.end()){
        contName = (*iter).second;
      }
      std::string classVersion("");
      iter = ops.find("classVersion");
      if(iter!=ops.end()){
        classVersion = (*iter).second;
      }
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      }
      bool allVersions = false;
      iter = ops.find("allVersions");
      if(iter!=ops.end()){
        allVersions = true;
      }

      if( !allVersions && theVersion.empty() && contName.empty() ){
        throw pool::Exception("Version or Container name not provided", "", "pool_retrieve_object_relational_mapping" );        
      }

      utility.initialize();
      utility.connect(theConnectionString,coral::ReadOnly);
      utility.session()->transaction().start(true);
      bool success = false;
      try{
        if(!theVersion.empty()){
          bool exists = utility.existsMapping(theVersion);
          if(exists){
            utility.retrieveMapping(theVersion,theMappingOutFile);
            success = true;
          } else {
            std::cout << "Mapping version " << theVersion << " not found in the database." << std::endl;
          }
        } else {
          if( !contName.empty() ){
            if(allVersions){ 
              if(!classVersion.empty())std::cout << "Warning: Class version selection will be ignored." << std::endl;
              utility.retrieveAllContainerMappings( contName,theMappingOutFile );
              success = true;
            } else{
              utility.retrieveContainerMapping( contName,theMappingOutFile, classVersion );
              success = true;
            }
          } else {
            utility.retrieveMapping(theMappingOutFile);
            success = true;          }
        }
        if( success ) std::cout << "Mapping dumped in file " << theMappingOutFile << std::endl;

        utility.session()->transaction().commit();
      } catch (const std::exception& e){
        utility.session()->transaction().rollback();
        std::cout << "ERROR: " << e.what() << std::endl;
        return 1;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

