#include "BaseContainerHandler.h"
#include "BaseContainerIteratorHandler.h"
#include "RelationalStorageServiceException.h"
#include "ObjectRelationalAccess/ObjectRelationalClassUtils.h"

pool::RelationalStorageService::BaseContainerHandler::BaseContainerHandler( const Reflex::Type& dictionary ):
  m_type( dictionary ),
  m_iteratorReturnType(),
  m_isAssociative( false ),
  m_collEnv(),
  m_collProxy(0)
{
  m_isAssociative = ObjectRelationalClassUtils::isTypeKeyedContainer( m_type );

  Reflex::Member method = m_type.MemberByName("createCollFuncTable");
  if(method){
#if ROOT_VERSION_CODE < ROOT_VERSION(5,21,3)
    m_collProxy = static_cast<Reflex::CollFuncTable*>(method.Invoke().Address());
#else
    method.Invoke(m_collProxy);
#endif
  }

  // find the iterator return type as the member type_value of the containers
  Reflex::Type valueType = ObjectRelationalClassUtils::containerValueType( m_type );
  m_iteratorReturnType = ObjectRelationalClassUtils::resolvedType( valueType );
  
}

pool::RelationalStorageService::BaseContainerHandler::~BaseContainerHandler(){
  if(m_collProxy){
    delete m_collProxy;
  }
}

size_t
pool::RelationalStorageService::BaseContainerHandler::size( const void* address )
{
  if(!m_collProxy){
    throw pool::RelationalStorageService::RelationalStorageServiceException( "Missing \"createCollFuncTable\" method for the container \"" +
                                                                             m_type.Name(Reflex::SCOPED|Reflex::FINAL) + "\"",
                                                                             "BaseContainerHandler" );
  }
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,22,0)
  m_collEnv.fObject = const_cast<void*>(address);
#else
  m_collEnv.object = const_cast<void*>(address);
#endif

  return *(static_cast<size_t*>(m_collProxy->size_func(&m_collEnv)));
}


pool::RelationalStorageService::IContainerIteratorHandler*
pool::RelationalStorageService::BaseContainerHandler::iterate( const void* address )
{
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,22,0)
  m_collEnv.fObject = const_cast<void*>(address);
#else
  m_collEnv.object = const_cast<void*>(address);
#endif

  if ( ! m_iteratorReturnType ) {
    throw pool::RelationalStorageService::RelationalStorageServiceException( "Missing the dictionary information for the value_type member of the container \"" +m_type.Name(Reflex::SCOPED|Reflex::FINAL) + "\"","BaseContainerHandler" );
  }
  return new pool::RelationalStorageService::BaseContainerIteratorHandler( m_collEnv,
                                                                           m_collProxy,
                                                                           m_iteratorReturnType );
}


void
pool::RelationalStorageService::BaseContainerHandler::appendNewElement( void* address, void* data )
{
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,22,0)
  m_collEnv.fObject = address;
  m_collEnv.fSize = 1;
  m_collEnv.fStart = data;
#else
  m_collEnv.object = address;
  m_collEnv.size = 1;
  m_collEnv.start = data;
#endif


  m_collProxy->feed_func(&m_collEnv);
}

void
pool::RelationalStorageService::BaseContainerHandler::clear( const void* address )
{
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,22,0)
  m_collEnv.fObject = const_cast<void*>(address);
#else
  m_collEnv.object = const_cast<void*>(address);
#endif
  m_collProxy->clear_func(&m_collEnv);
}

Reflex::Type
pool::RelationalStorageService::BaseContainerHandler::iteratorReturnType() const
{
  return m_iteratorReturnType;
}
