#ifndef BASECONTAINER_ITERATOR_HANDLER_H
#define BASECONTAINER_ITERATOR_HANDLER_H

#include "Reflex/Type.h"
#include "Reflex/Member.h"
#include "Reflex/Builder/CollectionProxy.h"
#include "IContainerIteratorHandler.h"
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
 typedef ROOT::Reflex::Type TypeH;
using namespace ROOT;
#else
 typedef Reflex::Type TypeH;
#endif

namespace pool {

  namespace RelationalStorageService {

    class BaseContainerIteratorHandler : virtual public IContainerIteratorHandler {
    public:
      /// Constructor
      BaseContainerIteratorHandler( const Reflex::Environ<long>& collEnv,
                                    Reflex::CollFuncTable* collProxy,
                                    const TypeH& iteratorReturnType );

      /// Destructor
      ~BaseContainerIteratorHandler();

      /// Increments itself
      void increment();

      /// Returns the current object
      void* object();

      /// Returns the return type of the iterator dereference method
      TypeH returnType() const;

    private:

      /// The return type of the iterator dereference method
      TypeH m_returnType;
      
      /// Structure containing parameters of the collection instance  
      Reflex::Environ<long> m_collEnv;

      /// Proxy of the generic collection
      Reflex::CollFuncTable* m_collProxy;

      /// Current element object pointer
      void* m_currentElement;
    };

  }

}

#endif
