#ifndef RELATIONALSTORAGESERVICE_OBJECTUPDATER_H
#define RELATIONALSTORAGESERVICE_OBJECTUPDATER_H

#include "ObjectWriter.h"

namespace pool {
  
  namespace RelationalStorageService {

    class ObjectUpdater : public ObjectWriter {
      
    public:
      /// Constructor
      ObjectUpdater( coral::ISchema& schema,
                     const Reflex::Type& dictionary,
                     MappingManager& mappingManager,
                     MappingSequence& mappingSequence,
                     pool::DbDatabase* databaseHandle );

      /// Destructor
      ~ObjectUpdater();

      /// Writes an object
      bool update( int oid,
                   const void* data,
                   pool::DataCallBack& callback );

      /// Updates an array - only for pool arrays there is an action
      bool updateArray( int oid,
                        pool::RelationalStorageService::ObjectRelationalDataBuffer& buffer,
                        const void* address );

      /// Updates a pool pointer
      bool updatePoolPointer( int oid,
                              ObjectRelationalDataBuffer& buffer,
                              const void* address,
                              pool::DataCallBack& callback );
      
    private:

      bool update( int oid,
                   ObjectRelationalDataBuffer& dataBuffer,
                   const void* data,
                   pool::DataCallBack& callback );

      bool deleteObjectDependencies( const pool::ObjectRelationalMappingElement& mapping,
                                     int oid, const std::string& topLevelTable, bool cascade=false);

      bool deleteArrayData( const pool::ObjectRelationalMappingElement& mapping,
                            int oid, int fromIndex, const std::string& topLevelTable );
      private:

      coral::ISchema& m_schema;
      
    };

  }

}

#endif
