// $Id: DbCursorImp.h,v 1.11 2007/11/20 10:39:13 frankb Exp $
//====================================================================
//
//  Package    : StorageSvc (The POOL project)
//
//  @author      M.Frank
//====================================================================
#ifndef POOL_DBCURSORIMP_H
#define POOL_DBCURSORIMP_H

/// Framework include files
#include "POOLCore/Token.h"
#include "StorageSvc/IDbCursor.h"
#include "StorageSvc/DbContainer.h"

/*
 *  POOL namespace declaration
 */
namespace pool    {

  // Forward declarations
  class DataCallBack;

  /** @class DbCursorImp DbCursorImp.h StorageSvc/DbCursorImp.h
    * 
    * Description:
    *
    *  The cursor allows to loop over objects residing 
    *  in a given container.
    *
    * @author   M.Frank
    * @version  1.0
    */
  class DbCursorImp : virtual public IDbCursor {
  protected:
    /// Current cursor for the loop
    Token m_token;
    /// Object type used for this scan
    const DbTypeInfo* m_type;
  public:
    /// Standard Constructor
    DbCursorImp(const DbTypeInfo* typ) : m_type(typ)  {   }
    /// Standard destructor
    virtual ~DbCursorImp()              {                 }
    /// Release cursor
    virtual void release()              { delete this;    }
    /// Access to cursor token
    virtual const Token* token() const  { return &m_token;}
    /// Retrieve next element
    virtual DbStatus next(DbContainer&  cntH,
                          DataCallBack* call,
                          DbAccessMode  mod)
    {
      m_token.oid().second++;
      return cntH.loadNext(call, m_token.oid(), mod);                    
    }
    /// Retrieve next element
    virtual DbStatus next(DbContainer&              cntH, 
                          DbObjectHandle<DbObject>& objH,
                          DbAccessMode              mod)
    {
      m_token.oid().second++;
      return cntH.loadNext(objH, m_token.oid(), m_type, mod);
    }
  };
}      // End namespace pool
#endif // POOL_DBCURSORIMP_H
