// $Id: DbOptionCallback.h,v 1.9 2007/11/20 16:43:06 frankb Exp $
//
//  Package    : pool/StorageSvc (The pool framework)
//
//  @author      M.Frank
//
//====================================================================
#ifndef POOL_DBOPTIONCALLBACK_H
#define POOL_DBOPTIONCALLBACK_H
// $Header: /cvs/PF/pool/StorageSvc/StorageSvc/DbOptionCallback.h,v 1.9 2007/11/20 16:43:06 frankb Exp $

// Framework include files
#include "StorageSvc/pool.h"

/*
 *   POOL namespace declaration
 */
namespace pool    {

  // Forward declarations
  class IDbOptionProxy;

  /** @class DbOptionCallback DbOptionCallback.h StorageSvc/DbOptionCallback.h 
    *
    * Description:
    *
    * Object to call an option setter to be used by the 
    * underlying domain, databases and containers.
    * The object setter must be declared on the stack.
    *
    * @author  M.Frank
    * @version 1.0
    */
  class DbOptionCallback    {
  public:
    /// Possible option type(s)
    enum OptionType  { 
      _SESSION_OPT, 
      _DOMAIN_OPT, 
      _DATABASE_OPT, 
      _CONTAINER_OPT, 
      _OBJECT_OPT,
      _SESSION_ONOPEN,
      _SESSION_ONCLOSE,
      _DOMAIN_ONOPEN,
      _DOMAIN_ONCLOSE,
      _DATABASE_ONOPEN,
      _DATABASE_ONCLOSE,
      _CONTAINER_ONOPEN,
      _CONTAINER_ONCLOSE
    };
  protected:
    /// Standard destructor
    DbOptionCallback();
    /// Default callback (does nothing)
    /** @param pObj      [IN]   Reference to object receiving options
      * @param typ       [IN]   Object type to receive options
      * @param name      [IN]   Object's name.
      *
      * @return Status code indicating success or failure.
      */
    virtual DbStatus setMyOptions(IDbOptionProxy* pObj, 
                                  OptionType typ, 
                                  const std::string& name);
  public:
    /// Constructor with initialization (from STACK ONLY!)
    DbOptionCallback(DbOptionCallback* actual_callback);
    /// Standard destructor
    virtual ~DbOptionCallback();
    /// Set options to me
    /** @param pObj      [IN]   Reference to object receiving options
      * @param typ       [IN]   Object type to receive options
      * @param name      [IN]   Object's name.
      *
      * @return Status code indicating success or failure.
      */
    static DbStatus getOptions(IDbOptionProxy* pObj, 
                               OptionType typ, 
                               const std::string& name);
  };
}      // End namespace pool
#endif // POOL_DBOPTIONCALLBACK_H
