// $Id: DbCallBack.cpp,v 1.23 2008/10/21 08:39:59 frankb Exp $
//====================================================================
//  DbCallBack object implementation
//--------------------------------------------------------------------
//
//  Package    : System (The POOL project)
//
//  Description: Generic data persistency
//
//  @author      M.Frank
//====================================================================

#include "StorageSvc/DbHeap.h"
#include "StorageSvc/DbReflex.h"
#include "StorageSvc/DbObject.h"
#include "StorageSvc/DbColumn.h"
#include "StorageSvc/DbCallBack.h"
#include "StorageSvc/DbTypeInfo.h"
#include "StorageSvc/DataHandler.h"
#include "StorageSvc/DbInstanceCount.h"
#include "Reflex/Type.h"
#include "Reflex/Object.h"
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

using namespace pool;
using namespace std;

typedef DbObjectHandle<DbObject> ObjHandle;
static DbInstanceCount::Counter* s_count = DbInstanceCount::getCounter(typeid(DbCallBack));

/// Standard Destructor
DbCallBack::DbCallBack( const DbContainer*              pCntH,
                        const DbObjectHandle<DbObject>* pObjH,
                        const DbTypeInfo*               pType)
: m_env(pCntH, pObjH, pType), 
  m_pObject(0),
  m_dataHandler(0)
{
  s_count->increment();
}

/// Standard Destructor
DbCallBack::DbCallBack(const DbCallBack& copy) 
: DataCallBack(copy), m_env(copy.m_env), 
  m_pObject(copy.m_pObject),
  m_dataHandler(copy.m_dataHandler)
{
  s_count->increment();
}

/// Standard Destructor
DbCallBack::~DbCallBack()  {
  s_count->decrement();
}

/// Returns the type of the transient object
DataCallBack::TypeH DbCallBack::transientType() const  {
  if ( m_env.typH ) {
    return m_env.typH->clazz();
  }
  return TypeH();
}


/// Callback when a read sequence should be started
DbStatus DbCallBack::start(CallType action_type, void* pObj, void** data_pointer)  {
  switch(action_type)  {
  case GET:
    if ( m_env.typH && m_env.objH && m_env.cntH )  {
      ObjHandle*   pHdl = const_cast<ObjHandle*>(m_env.objH);
      DbContainer* pCnt = const_cast<DbContainer*>(m_env.cntH);
      TypeH cl = m_env.typH->clazz();
      if ( cl )   {
        DbObject* ptr = DbHeap::allocate(cl.SizeOf(),pCnt,0,pHdl);
        if ( ptr )   {
          Reflex::Object o = cl.Construct(TypeH(0),vector<void*>(), ptr);
          pHdl->_setObject(ptr);
          m_pObject = ptr;
          *data_pointer = m_pObject;
          return Success;
        }
      }
    }
    return Error;
  case PUT:
    m_pObject = pObj;
    *data_pointer = m_pObject;
    return Success;
  default:
    return Error;
  }
}

/// Callback to retrieve the absolute address of a column
DbStatus DbCallBack::bind(CallType        action_type,    // GET or PUT
                          const DbColumn* col_identifier, // Col ID
                          int          /* col_number */,  // Col number
                          void*           context,        // Data context
                          void**          data_pointer)   // data pointer
{
  switch(action_type)  {
  case GET:
  case PUT:
    *data_pointer = ((char*)context)+col_identifier->offset();
    return Success;
  default:
    return Error;
  }
}

/// Callback when a read sequence should be started
DbStatus DbCallBack::end(CallType /* action */, void* /* context */) {
  return Success;
}

/// Callback to handle on the fly object manipulation during save/load operations.
DbStatus DbCallBack::notify(CallType  action_type,    // GET or PUT
                            int       data_type,      // data type
                            TypeH     reflection,     // Reflection information
                            void*     object_ptr,     // object ptr
                            void*     data)           // data pointer
{
  switch( action_type )   {
  case GET:
    if ( m_dataHandler ) {
      return m_dataHandler->onLoad(data_type, reflection, object_ptr, data);
    }
  case PUT:
    if ( m_dataHandler ) {
      return m_dataHandler->onSave(data_type, reflection, object_ptr, data);
    }
  default:
    break;
  }
  return Success;
}
