#include "StorageSvc/DbObjectCallBack.h"
#include "StorageSvc/DataHandler.h"
#include "StorageSvc/DbColumn.h"
using namespace pool;

DbStatus DbObjectCallBack::start(CallType action_type,void* data,void** context)  {
  switch(action_type) {
  case GET:
    {
      // Do not allocate space for the persistent data buffer.
      // The back end implementation will do that.
      m_objectData = 0;
    }
    break;
  case PUT:
    // Check if the data corresponds to the transient data pointer
    if ( data != m_objectData ) {
      // Input data is different from the transient data
      return Error;
    }
    // No need to allocate space for the persistent data.
    // The supplied transient data buffer is used as
    // the persistent data buffer
    break;
  default:
    return Error;
  };
  *context = const_cast< void* >( m_objectData );
  return Success;
}


DbStatus DbObjectCallBack::bind( CallType action_type,
                                 const DbColumn* col_identifier,
                                 int /* col_number */,
                                 void* context,
                                 void** data )
{
  switch(action_type)  {
  case GET:
    // For reading we can only bind the starting address of the persistent data buffer.
    *data = &m_objectData;
    break;
  case PUT:
    /* This works for ROOT Trees but not for ROOT keys !!!
    if ( context != m_objectData ) {
      std::cerr << "Context different than persistent data" << std::endl;
      return Error;
    }
    */
    *data = static_cast< char* >( context ) + col_identifier->offset();
    break;
  default:
    return Error;
  };
  return Success;
}

DbStatus DbObjectCallBack::end( CallType action_type, void* /* context */ ) {
  switch(action_type)  {
  case GET:
    // Reading: we do not need to create a transient buffer and copy
    // the relevant data from the transient part.
    // We do not release the memory of the persistent buffer, since this
    // is the resulting transient one.
    break;
  case PUT:
    // Writing: we do not release the persistent data buffer, since this
    // is the input transient one.
    break;
  default:
    return Error;
  };
  return Success;
}


DbStatus DbObjectCallBack::notify( CallType action_type,
                                   int      data_type,
                                   TypeH    reflection,
                                   void*    object_ptr,
                                   void*    data )
{ // Delegate the call to the data handler.
  if ( m_dataHandler ) {
    switch( action_type )   {
    case GET:
      return m_dataHandler->onLoad(data_type, reflection, object_ptr, data);
    case PUT:
      return m_dataHandler->onSave(data_type, reflection, object_ptr, data);
    default:
      return Error;
    };
  }
  return Success;
}
