// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------

#include "xercesc/dom/deprecated/DOMString.hpp"
#include "xercesc/sax/SAXParseException.hpp"
#include "CoralBase/MessageStream.h"
#include "XMLDOMErrorMessanger.h"

#include <iostream>
#include <string>
#include <stdexcept>

// Global streaming operator for DOMString is defined in DOMPrint.cpp
//extern ostream& operator<<(ostream& target, const DOMString& s);


namespace pool{  
  inline std::string _toString(const DOMString &toTranscode)
  {
    char * buff = toTranscode.transcode();
    std::string tmp(buff);
    XMLString::release(&buff);
    return tmp;
  }
}

pool::XMLDOMErrorMessanger::XMLDOMErrorMessanger() :
  fSawErrors(false)
{}

void
pool::XMLDOMErrorMessanger::warning(const SAXParseException&)
{
  //
  // Ignore all warnings.
  //
}

void
pool::XMLDOMErrorMessanger::error(const SAXParseException& toCatch)
{
  std::string message = pool::_toString(DOMString(toCatch.getMessage()));
  
  if (message.find("The values for attribute 'name' must be names or name tokens")!=std::string::npos ||
      message.find("The values for attribute 'ID' must be names or name tokens")!=std::string::npos   ||
      message.find("for attribute 'name' must be Name or Nmtoken")!=std::string::npos                 ||
      message.find("for attribute 'ID' must be Name or Nmtoken")!=std::string::npos
    )
  {
    return;
  }

  fSawErrors = true;
  coral::MessageStream xmllog( "XMLDOMErrorMessanger");
  xmllog <<coral::Error  << "Error at file \""  << pool::_toString(DOMString(toCatch.getSystemId()))
         << "\", line " << toCatch.getLineNumber()
         << ", column " << toCatch.getColumnNumber()
         << "\n   Message: "    << message
         << coral::MessageStream::endmsg;
}

void
pool::XMLDOMErrorMessanger::fatalError(const SAXParseException& toCatch)
{
  fSawErrors = true;
  coral::MessageStream xmllog( "XMLDOMErrorMessanger");
  xmllog<< coral::Fatal << "Fatal Error at file \"" << pool::_toString(DOMString(toCatch.getSystemId()))
        << "\", line "      << toCatch.getLineNumber()
        << ", column "      << toCatch.getColumnNumber()
        << "\n   Message: "         << pool::_toString(DOMString(toCatch.getMessage())) << coral::MessageStream::endmsg;
  //  coral::MessageStream::Status s(coral::MessageStream::Status::ERROR,0,10);
  throw std::runtime_error( "Standard pool exception : Fatal Error on the DOM Parser" );
}

void
pool::XMLDOMErrorMessanger::resetErrors()
{
  // No-op in this case
}
